Package: formatdown
Title: Formatting Numbers in 'rmarkdown' Documents
Version: 0.2.0
Language: en-US
Authors@R: person("Richard", "Layton", email="graphdoctor@gmail.com", role=c("aut", "cre"))
Description: Provides a small set of tools for formatting numbers in R-markdown 
    documents. Convert a numerical vector to character strings in power-of-ten 
    form, decimal form, or measurement-units form; convert symbols for chemical 
    isotopes from hyphenated form to nuclear form; all are math-delimited for 
    rendering as inline equations. Can also convert text into math-delimited 
    text to match the font face and size of math-delimited numbers. Useful for 
    rendering single numbers in inline R code chunks and for rendering columns 
    in tables.   
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: TRUE
LazyDataCompression: bzip2
RoxygenNote: 7.3.3
Imports: checkmate, data.table, settings, units, wrapr
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
URL: https://github.com/graphdr/formatdown/,
        https://graphdr.github.io/formatdown/,
        https://CRAN.R-project.org/package=formatdown
BugReports: https://github.com/graphdr/formatdown/issues
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-05-02 22:56:59 UTC; layto
Author: Richard Layton [aut, cre]
Maintainer: Richard Layton <graphdoctor@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-02 23:10:03 UTC
Built: R 4.5.2; ; 2026-05-02 23:57:20 UTC; unix

Package: formatdown
Title: Formatting Numbers in 'rmarkdown' Documents
Version: 0.2.0
Language: en-US
Authors@R: person("Richard", "Layton", email="graphdoctor@gmail.com", role=c("aut", "cre"))
Description: Provides a small set of tools for formatting numbers in R-markdown 
    documents. Convert a numerical vector to character strings in power-of-ten 
    form, decimal form, or measurement-units form; convert symbols for chemical 
    isotopes from hyphenated form to nuclear form; all are math-delimited for 
    rendering as inline equations. Can also convert text into math-delimited 
    text to match the font face and size of math-delimited numbers. Useful for 
    rendering single numbers in inline R code chunks and for rendering columns 
    in tables.   
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: TRUE
LazyDataCompression: bzip2
RoxygenNote: 7.3.3
Imports: checkmate, data.table, settings, units, wrapr
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
URL: https://github.com/graphdr/formatdown/,
        https://graphdr.github.io/formatdown/,
        https://CRAN.R-project.org/package=formatdown
BugReports: https://github.com/graphdr/formatdown/issues
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-05-02 22:56:59 UTC; layto
Author: Richard Layton [aut, cre]
Maintainer: Richard Layton <graphdoctor@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-02 23:10:03 UTC
Built: R 4.6.0; ; 2026-05-03 01:04:36 UTC; unix

# Generated by roxygen2: do not edit by hand

export(format_dcml)
export(format_engr)
export(format_nucl)
export(format_numbers)
export(format_sci)
export(format_text)
export(formatdown_options)
importFrom(checkmate,assert)
importFrom(checkmate,assert_choice)
importFrom(checkmate,assert_disjunct)
importFrom(checkmate,assert_true)
importFrom(checkmate,check_class)
importFrom(checkmate,qassert)
importFrom(checkmate,test_class)
importFrom(data.table,"%between%")
importFrom(data.table,"%like%")
importFrom(data.table,":=")
importFrom(data.table,copy)
importFrom(data.table,fcase)
importFrom(data.table,fifelse)
importFrom(data.table,setDT)
importFrom(data.table,tstrsplit)
importFrom(settings,options_manager)
importFrom(settings,reset)
importFrom(settings,stop_if_reserved)
importFrom(units,as_units)
importFrom(units,deparse_unit)
importFrom(wrapr,"%?%")
importFrom(wrapr,stop_if_dot_args)

Unit tests    

Richard A. Layton    
2022-11-16    


In writing defensive programming unit tests for formatdown, I use the packages:

- checkmate, to write the runtime assertions to check function arguments. These assertions are in the function files in the `formatdown/R/` directory.   

- tinytest, to write the unit tests that check function arguments before the function executes. The test files are in the directory `formatdown/inst/tinytest/`. 

<!-- Edit README.Rmd (not README.md) -->

# formatdown <img src="man/figures/logo.png" align="right">

Formatting Numbers in R Markdown Documents

<!-- badges: start -->

[![](https://www.r-pkg.org/badges/version/formatdown)](https://cran.r-project.org/package=formatdown)
[![R-CMD-check](https://github.com/graphdr/formatdown/actions/workflows/check-standard.yaml/badge.svg)](https://github.com/graphdr/formatdown/actions/workflows/check-standard.yaml)
[![Codecov test
coverage](https://codecov.io/gh/graphdr/formatdown/branch/main/graph/badge.svg)](https://app.codecov.io/gh/graphdr/formatdown?branch=main)
[![Lifecycle](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

Provides a small set of tools for formatting numbers in R markdown
documents (file type `.Rmd` or `.qmd`). Converts a numerical vector to
*character strings* in power-of-ten form, decimal form, or
measurement-units form; all are math-delimited within quotation marks
for rendering as inline equations. Useful for rendering numerical
scalars using inline R code chunks or for rendering numerical columns in
tables.

## Introduction

In professional technical prose, large and small numbers are generally
typeset using powers of ten notation. For example, Planck’s constant
would be typeset as $6.63 \times 10^{-34}$ J/Hz rather than the familiar
forms we use in communicating with computers, such as `6.63*10^-34` or
`6.63E-34`.

The functions in this package help an author of an R markdown document
convert large and small numbers to character strings, formatted using
powers-of-ten notation. In addition, decimal numbers and text can be
formatted with the same font face and size as the power-of-ten numbers
for a consistent typeface across all columns of a data table.

Formatting tools include:

**`format_numbers()`**  
Convert a numeric vector to a math-delimited character vector in which
the numbers can be formatted in scientific or engineering power-of-ten
notation or in decimal form. Three convenience functions that wrap
`format_numbers()` are provided as well: `format_sci()` for scientific
notation, `format_engr()` for engineering notation, and `format_dcml()`
for decimal notation.

**`format_text()`**  
Convert a character vector to math-delimited character vector. Useful
for creating a consistent typeface across all columns of a table.

**`format_nucl()`**  
Convert a character vector of chemical isotopes in hyphenated form to a
math-delimited character vector in nuclear form: each element symbol
with its mass number superscript and the optional atomic number
subscript.

**`formatdown_options()`**  
Global options are provided for arguments that users would likely prefer
to set once in a document instead of repeating in every function call.
For example, some users prefer a comma decimal marker (“,”) throughout a
document.

## Usage

``` r
# Packages
library("formatdown")
library("data.table")
library("knitr")
```

*Scalar values.*   Typically rendered inline:

``` r
x <- 101300

# Scientific notation
format_numbers(x, digits = 4, format = "sci")
#> [1] "$1.013 \\times 10^{5}$"

# Engineering notation
format_numbers(x, digits = 4, format = "engr")
#> [1] "$101.3 \\times 10^{3}$"

# Decimal notation
format_numbers(x, digits = 4, format = "dcml")
#> [1] "$101300$"

# With measurement units
units(x) <- "Pa"
units(x) <- "hPa"
format_dcml(x)
#> [1] "$1013\\ \\mathrm{hPa}$"
```

which, in an `.Rmd` or `.qmd` output document, are rendered using inline
R code as

|      Format |           Rendered as |
|------------:|----------------------:|
|  scientific | $1.013 \times 10^{5}$ |
| engineering | $101.3 \times 10^{3}$ |
|     decimal |              $101300$ |
|       units |  $1013\ \mathrm{hPa}$ |

<br>

*Data frame*.   Typically rendered in a table. We independently format
columns from the `metals` data frame included with formatdown.

``` r
# View the data set
metals
#>            metal  dens  thrm_exp thrm_cond  elast_mod
#>           <char> <num>     <num>     <num>      <num>
#> 1: aluminum 6061  2700 2.430e-05    155.77 7.3084e+10
#> 2:        copper  8900 1.656e-05    392.88 1.1721e+11
#> 3:          lead 11340 5.274e-05     37.04 1.3790e+10
#> 4:      platinum 21450 9.000e-06     69.23 1.4686e+11
#> 5:    steel 1020  7850 1.134e-05     46.73 2.0684e+11
#> 6:      titanium  4850 9.360e-06      7.44 1.0204e+11

# First column in text format
DT <- copy(metals)
DT$metal <- format_text(DT$metal)

# Density and thermal conductivity in decimal form
cols_we_want <- c("dens", "thrm_cond")
DT[, cols_we_want] <- lapply(DT[, ..cols_we_want], function(x) format_dcml(x, 3))

# Thermal expansion in engineering format
DT$thrm_exp <- format_engr(DT$thrm_exp, 3)

# Elastic modulus in units form
units(DT$elast_mod) <- "Pa"
units(DT$elast_mod) <- "GPa"
DT$elast_mod <- format_dcml(DT$elast_mod, 3)

# Render in document
knitr::kable(DT,
  align = "r",
  caption = "Table 1: Properties of metals.",
  col.names = c("Metal", "Density [kg/m$^3$]", "Therm. expan. [m/m K$^{-1}$]", "Therm. cond. [W/m K$^{-1}$]", "Elastic modulus")
)
```

| Metal | Density \[kg/m$^3$\] | Therm. expan. \[m/m K$^{-1}$\] | Therm. cond. \[W/m K$^{-1}$\] | Elastic modulus |
|---:|---:|---:|---:|---:|
| $\mathrm{aluminum\ 6061}$ | $2700$ | $24.3 \times 10^{-6}$ | $156$ | $73.1\ \mathrm{GPa}$ |
| $\mathrm{copper}$ | $8900$ | $16.6 \times 10^{-6}$ | $393$ | $117\ \mathrm{GPa}$ |
| $\mathrm{lead}$ | $11300$ | $52.7 \times 10^{-6}$ | $37.0$ | $13.8\ \mathrm{GPa}$ |
| $\mathrm{platinum}$ | $21400$ | $9.00 \times 10^{-6}$ | $69.2$ | $147\ \mathrm{GPa}$ |
| $\mathrm{steel\ 1020}$ | $7850$ | $11.3 \times 10^{-6}$ | $46.7$ | $207\ \mathrm{GPa}$ |
| $\mathrm{titanium}$ | $4850$ | $9.36 \times 10^{-6}$ | $7.44$ | $102\ \mathrm{GPa}$ |

Table 1: Properties of metals.

<br>

*Options*.   For users who prefer a comma as the decimal mark, the
argument can be set once using `formatdown_options()`. If desired, we
can also change the multiplication symbol to a half-high dot.

``` r
formatdown_options(decimal_mark = ",", multiply_mark = "\\cdot")
```

Using the same code as above to format the metals data yields,

| Metal | Density \[kg/m$^3$\] | Therm. expan. \[m/m K$^{-1}$\] | Therm. cond. \[W/m K$^{-1}$\] | Elastic modulus |
|---:|---:|---:|---:|---:|
| $\mathrm{aluminum\ 6061}$ | $2700$ | $24,3 \cdot 10^{-6}$ | $156$ | $73,1\ \mathrm{GPa}$ |
| $\mathrm{copper}$ | $8900$ | $16,6 \cdot 10^{-6}$ | $393$ | $117\ \mathrm{GPa}$ |
| $\mathrm{lead}$ | $11300$ | $52,7 \cdot 10^{-6}$ | $37,0$ | $13,8\ \mathrm{GPa}$ |
| $\mathrm{platinum}$ | $21400$ | $9,00 \cdot 10^{-6}$ | $69,2$ | $147\ \mathrm{GPa}$ |
| $\mathrm{steel\ 1020}$ | $7850$ | $11,3 \cdot 10^{-6}$ | $46,7$ | $207\ \mathrm{GPa}$ |
| $\mathrm{titanium}$ | $4850$ | $9,36 \cdot 10^{-6}$ | $7,44$ | $102\ \mathrm{GPa}$ |

Table 2: Changing the decimal mark

<br>

To return to the default values,

``` r
formatdown_options(reset = TRUE)
```

## Installation

Install from CRAN.

``` r
install.packages("formatdown")
```

The development version can be installed from GitHub. I suggest using
the “pak” package:

``` r
pak::pkg_install("graphdr/formatdown")
```

## Requirements

- [`R`](https://www.r-project.org/) (\>= 3.5.0)
- [`data.table`](https://rdatatable.gitlab.io/data.table/) (\>= 1.9.8)

## Contributing

To provide feedback or report a bug,

- Use the GitHub <a href="https://github.com/graphdr/formatdown/issues">
  Issues</a> page.

To contribute to formatdown,

- Please clone this repo locally.  
- Commit your contribution on a separate branch.
- Submit a pull request

Participation in this open source project is subject to a [Code of
Conduct](https://graphdr.github.io/formatdown/CONDUCT.html).

## Software credits

- [`R`](https://www.r-project.org/) and [`RStudio`](https://posit.co/)
  for the working environment  
- [`rmarkdown`](https://CRAN.R-project.org/package=rmarkdown) and
  [`knitr`](https://CRAN.R-project.org/package=knitr) for authoring
  tools  
- [`data.table`](https://CRAN.R-project.org/package=data.table) for its
  programmable syntax  
- [`units`](https://CRAN.R-project.org/package=units) for handling
  physical units
- [`wrapr`](https://CRAN.R-project.org/package=wrapr),
  [`checkmate`](https://CRAN.R-project.org/package=checkmate), and
  [`tinytest`](https://CRAN.R-project.org/package=tinytest) for
  programming tools
- [`devtools`](https://CRAN.R-project.org/package=devtools) and
  [`pkgdown`](https://CRAN.R-project.org/package=pkgdown) for package
  building

<!-- Edit README.Rmd (not README.md) -->
<!-- Correcting for problems with LaTeX math in the README file only -->
<style type="text/css">
.math {
  font-size: small;
}
</style>

# formatdown <img src="man/figures/logo.png" align="right">

Formatting Numbers in R Markdown Documents

<!-- badges: start -->

[![](https://www.r-pkg.org/badges/version/formatdown)](https://cran.r-project.org/package=formatdown)
[![R-CMD-check](https://github.com/graphdr/formatdown/actions/workflows/check-standard.yaml/badge.svg)](https://github.com/graphdr/formatdown/actions/workflows/check-standard.yaml)
[![Codecov test
coverage](https://codecov.io/gh/graphdr/formatdown/branch/main/graph/badge.svg)](https://app.codecov.io/gh/graphdr/formatdown?branch=main)
[![Lifecycle](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

Provides a small set of tools for formatting numbers in R markdown
documents (file type `.Rmd` or `.qmd`). Converts a numerical vector to
*character strings* in power-of-ten form, decimal form, or
measurement-units form; all are math-delimited within quotation marks
for rendering as inline equations. Useful for rendering numerical
scalars using inline R code chunks or for rendering numerical columns in
tables.

## Introduction

In professional technical prose, large and small numbers are generally
typeset using powers of ten notation. For example, Planck’s constant
would be typeset as $6.63 \times 10^{-34}\>\mathrm{J\,Hz^{-1}}$ rather
than the familiar forms we use in communicating with computers, such as
`6.63*10^-34` or `6.63E-34`.

The functions in this package help an author of an R markdown document
convert large and small numbers to character strings, formatted using
powers-of-ten notation. In addition, decimal numbers and text can be
formatted with the same font face and size as the power-of-ten numbers
for a consistent typeface across all columns of a data table.

Formatting tools include:

**`format_numbers()`**  
Convert a numeric vector to a math-delimited character vector in which
the numbers can be formatted in scientific or engineering power-of-ten
notation or in decimal form.

**`format_sci()`**  
Convenience function. A wrapper around `format_numbers()` for scientific
notation.

**`format_engr()`**  
Convenience function. A wrapper around `format_numbers()` for
engineering notation.

**`format_dcml()`**  
Convenience function. A wrapper around `format_numbers()` for decimal
notation.

**`format_text()`**  
Convert a character vector to math-delimited character vector. Useful
for creating a consistent typeface across all columns of a table.

**`formatdown_options()`**  
Global options are provided for arguments that users would likely prefer
to set once in a document instead of repeating in every function call.
For example, some users prefer a comma decimal marker (“,”) throughout a
document.

## Usage

``` r
# Packages
library("formatdown")
library("data.table")
library("knitr")
```

*Scalar values.*   Typically rendered inline:

``` r
x <- 101300

# Scientific notation
format_numbers(x, digits = 4, format = "sci")
#> [1] "$1.013 \\times 10^{5}$"

# Engineering notation
format_numbers(x, digits = 4, format = "engr")
#> [1] "$101.3 \\times 10^{3}$"

# Decimal notation
format_numbers(x, digits = 4, format = "dcml")
#> [1] "$101300$"

# With measurement units
units(x) <- "Pa"
units(x) <- "hPa"
format_dcml(x)
#> [1] "$1013\\>\\mathrm{hPa}$"
```

which, in an `.Rmd` or `.qmd` output document, are rendered using inline
R code as

|      Format |           Rendered as |
|------------:|----------------------:|
|  scientific | $1.013 \times 10^{5}$ |
| engineering | $101.3 \times 10^{3}$ |
|     decimal |              $101300$ |
|       units |  $1013\>\mathrm{hPa}$ |

<br>

*Data frame*.   Typically rendered in a table. We independently format
columns from the `metals` data frame included with formatdown.

``` r
# View the data set
metals
#>            metal  dens  thrm_exp thrm_cond  elast_mod
#>           <char> <num>     <num>     <num>      <num>
#> 1: aluminum 6061  2700 2.430e-05    155.77 7.3084e+10
#> 2:        copper  8900 1.656e-05    392.88 1.1721e+11
#> 3:          lead 11340 5.274e-05     37.04 1.3790e+10
#> 4:      platinum 21450 9.000e-06     69.23 1.4686e+11
#> 5:    steel 1020  7850 1.134e-05     46.73 2.0684e+11
#> 6:      titanium  4850 9.360e-06      7.44 1.0204e+11

# First column in text format
DT <- copy(metals)
DT$metal <- format_text(DT$metal)

# Density and thermal conductivity in decimal form
cols_we_want <- c("dens", "thrm_cond")
DT[, cols_we_want] <- lapply(DT[, ..cols_we_want], function(x) format_dcml(x, 3))

# Thermal expansion in engineering format
DT$thrm_exp <- format_engr(DT$thrm_exp, 3)

# Elastic modulus in units form
units(DT$elast_mod) <- "Pa"
units(DT$elast_mod) <- "GPa"
DT$elast_mod <- format_dcml(DT$elast_mod, 3)

# Render in document
knitr::kable(DT, align = "r", caption = "Table 1: Properties of metals.", col.names = c("Metal",
    "Density [kg/m$^3$]", "Therm. expan. [m/m K$^{-1}$]", "Therm. cond. [W/m K$^{-1}$]",
    "Elastic modulus"))
```

|                     Metal | Density \[kg/m$^3$\] | Therm. expan. \[m/m K$^{-1}$\] | Therm. cond. \[W/m K$^{-1}$\] |      Elastic modulus |
|--------------------------:|---------------------:|-------------------------------:|------------------------------:|---------------------:|
| $\mathrm{aluminum\>6061}$ |               $2700$ |          $24.3 \times 10^{-6}$ |                         $156$ | $73.1\>\mathrm{GPa}$ |
|         $\mathrm{copper}$ |               $8900$ |          $16.6 \times 10^{-6}$ |                         $393$ |  $117\>\mathrm{GPa}$ |
|           $\mathrm{lead}$ |              $11300$ |          $52.7 \times 10^{-6}$ |                        $37.0$ | $13.8\>\mathrm{GPa}$ |
|       $\mathrm{platinum}$ |              $21400$ |          $9.00 \times 10^{-6}$ |                        $69.2$ |  $147\>\mathrm{GPa}$ |
|    $\mathrm{steel\>1020}$ |               $7850$ |          $11.3 \times 10^{-6}$ |                        $46.7$ |  $207\>\mathrm{GPa}$ |
|       $\mathrm{titanium}$ |               $4850$ |          $9.36 \times 10^{-6}$ |                        $7.44$ |  $102\>\mathrm{GPa}$ |

Table 1: Properties of metals.

<br>

*Options*.   For users who prefer a comma as the decimal mark, the
argument can be set once using `formatdown_options()`,

``` r
formatdown_options(decimal_mark = ",")
```

Using the same code as above to format the metals data yields,

|                     Metal | Density \[kg/m$^3$\] | Therm. expan. \[m/m K$^{-1}$\] | Therm. cond. \[W/m K$^{-1}$\] |      Elastic modulus |
|--------------------------:|---------------------:|-------------------------------:|------------------------------:|---------------------:|
| $\mathrm{aluminum\>6061}$ |               $2700$ |          $24,3 \times 10^{-6}$ |                         $156$ | $73,1\>\mathrm{GPa}$ |
|         $\mathrm{copper}$ |               $8900$ |          $16,6 \times 10^{-6}$ |                         $393$ |  $117\>\mathrm{GPa}$ |
|           $\mathrm{lead}$ |              $11300$ |          $52,7 \times 10^{-6}$ |                        $37,0$ | $13,8\>\mathrm{GPa}$ |
|       $\mathrm{platinum}$ |              $21400$ |          $9,00 \times 10^{-6}$ |                        $69,2$ |  $147\>\mathrm{GPa}$ |
|    $\mathrm{steel\>1020}$ |               $7850$ |          $11,3 \times 10^{-6}$ |                        $46,7$ |  $207\>\mathrm{GPa}$ |
|       $\mathrm{titanium}$ |               $4850$ |          $9,36 \times 10^{-6}$ |                        $7,44$ |  $102\>\mathrm{GPa}$ |

Table 2: Changing the decimal mark

<br>

To return to the default values,

``` r
formatdown_options(reset = TRUE)
```

## Installation

Install from CRAN.

``` r
install.packages("formatdown")
```

The development version can be installed from GitHub. I suggest using
the “pak” package:

``` r
pak::pkg_install("graphdr/formatdown")
```

## Requirements

- [`R`](https://www.r-project.org/) (\>= 3.5.0)
- [`data.table`](https://rdatatable.gitlab.io/data.table/) (\>= 1.9.8)

## Contributing

To provide feedback or report a bug,

- Use the GitHub <a href="https://github.com/graphdr/formatdown/issues">
  Issues</a> page.

To contribute to formatdown,

- Please clone this repo locally.  
- Commit your contribution on a separate branch.
- Submit a pull request

Participation in this open source project is subject to a [Code of
Conduct](https://graphdr.github.io/formatdown/CONDUCT.html).

## Software credits

- [`R`](https://www.r-project.org/) and [`RStudio`](https://posit.co/)
  for the working environment  
- [`rmarkdown`](https://CRAN.R-project.org/package=rmarkdown) and
  [`knitr`](https://CRAN.R-project.org/package=knitr) for authoring
  tools  
- [`data.table`](https://CRAN.R-project.org/package=data.table) for its
  programmable syntax  
- [`units`](https://CRAN.R-project.org/package=units) for handling
  physical units
- [`wrapr`](https://CRAN.R-project.org/package=wrapr),
  [`checkmate`](https://CRAN.R-project.org/package=checkmate), and
  [`tinytest`](https://CRAN.R-project.org/package=tinytest) for
  programming tools
- [`devtools`](https://CRAN.R-project.org/package=devtools) and
  [`pkgdown`](https://CRAN.R-project.org/package=pkgdown) for package
  building

Unit tests with tinytest 

https://github.com/markvanderloo/tinytest

We use the tinytest R package for unit tests. In tinytest, tests 
are scripts interspersed with statements that perform checks. 

Package testing is set up as follows: 

1. Test scripts are located in formatdown/inst/tinytest/.   
   Filenames start with `test_`, for example, `test_format_power.R`.  
   Usually one file per function.  
  
2. The formatdown/tests/ directory contains an R file named tinytest.R that 
   contains the following script:  

     if (requireNamespace("tinytest", quietly = TRUE)){
       tinytest::test_package("formatdown" )
     }

3. In the DESCRIPTION file, tinytest is added to Suggests:. 
  
formatdown users can run the tinytest tests themselves by 
installing tinytest and running: 
 
    tinytest::test_package("formatdown")

For detailed information including test functions, see 

    vignette("using_tinytest", package = "tinytest")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  tidy = TRUE
)

# to knit "child" Rmd files
knitr::opts_knit$set(root.dir = "../")

library(formatdown)
library(data.table)
library(knitr)

options(
  datatable.print.nrows = 15,
  datatable.print.topn = 3,
  datatable.print.class = TRUE
)

## -----------------------------------------------------------------------------
formatdown_options("size", "decimal_mark")

## -----------------------------------------------------------------------------
# Set
formatdown_options(size = "large", decimal_mark = ",")

# Examine result
formatdown_options("size", "decimal_mark")

## -----------------------------------------------------------------------------
# Set to defaults
formatdown_options(reset = TRUE)

# Examine result
formatdown_options("size", "decimal_mark")

## -----------------------------------------------------------------------------
x <- 101300
txt <- "Hello world!"

# 1. Numeric input, default delimiters
format_dcml(x)

# 2. Numeric input, alternate delimiters
format_dcml(x, delim = "\\(")

# 3. Character input, default delimiters
format_text(txt)

# 4. Character input, alternate delimiters
format_text(txt, delim = "\\(")

## -----------------------------------------------------------------------------
# 5. Numeric input
format_dcml(x, size = "scriptsize")

# 6. Numeric input
format_dcml(x, size = "small")

# 7. Power-of-ten number using LaTeX-style size markup
format_sci(6.0221e+23, size = "\\small")

# 8. Character input, default size
format_text(txt)

# 9. Character input
format_text(txt, size = "large")

## -----------------------------------------------------------------------------
x <- pi
y <- 5.3e+3
z <- "The cat"

scriptsize <- c(
  format_dcml(x, 5, size = "scriptsize"),
  format_sci(y, 1, size = "scriptsize"),
  format_text(z, size = "scriptsize")
)
small <- c(
  format_dcml(x, 5, size = "small"),
  format_sci(y, 1, size = "small"),
  format_text(z, size = "small")
)
normalsize <- c(
  format_dcml(x, 5, size = "normalsize"),
  format_sci(y, 1, size = "normalsize"),
  format_text(z, size = "normalsize")
)
large <- c(
  format_dcml(x, 5, size = "large"),
  format_sci(y, 1, size = "large"),
  format_text(z, size = "large")
)
huge <- c(
  format_dcml(x, 5, size = "huge"),
  format_sci(y, 1, size = "huge"),
  format_text(z, size = "huge")
)

DT <- data.table(scriptsize, small, normalsize, large, huge)
knitr::kable(DT, align = "r")

## -----------------------------------------------------------------------------
# 10. Decimal markup
x <- pi
format_dcml(x, 5, decimal_mark = ",")

# 11. Power-of-ten markup
y <- 1.602176634e-19
format_sci(y, 5, decimal_mark = ",")

## -----------------------------------------------------------------------------
w <- 1013
x <- 101300
y <- 0.002456
z <- x + y

# 12. 4-digit number, no space
format_dcml(w)

# 13. 4-digit number, with space
format_dcml(w, big_mark = "thin")

# 14. Group digits to the left of the decimal
format_dcml(x, big_mark = "thin")

# 15. Group digits to the right of the decimal
format_dcml(y, small_mark = "\\\\,")

# 16. Change the small interval
format_dcml(y, small_mark = "\\\\,", small_interval = 3)

# 17. Group digits to the left and right of the decimal
format_dcml(z, 12, big_mark = "thin", small_mark = "thin")

## -----------------------------------------------------------------------------
# 18. Character input, default space "\>"
format_text(txt, whitespace = "\\\\>")

# 19. Character input, alternate space "\:"
format_text(txt, whitespace = "\\\\:")

# 20. Character input, alternate space "\ "
format_text(txt, whitespace = "\\\\ ")

## -----------------------------------------------------------------------------
formatdown_options(size = "small")

## -----------------------------------------------------------------------------
# Set options
formatdown_options(decimal_mark = ",", big_mark = "thin", small_mark = "thin")

# Use water data included with formatdown
DT <- copy(water)[1:6]

# Examine the data frame
DT[]

# Routine decimal formatting
DT$temp <- format_dcml(DT$temp)
DT$dens <- format_dcml(DT$dens)

# Omit big_mark spacing for 4 digits
DT$sp_wt <- format_dcml(DT$sp_wt, 4, big_mark = "")

# Set significant digits (viscosity) to achieve a consistent string length
DT[visc >= 0.001, temp_visc := format_dcml(visc, 5)]
DT[visc < 0.001, temp_visc := format_dcml(visc, 4)]
DT[, visc := temp_visc]
DT[, temp_visc := NULL]

# Will appear with big_mark spacing, change from Pa to kPa
DT$bulk_mod_kPa <- format_dcml(DT$bulk_mod / 1000, 4)
DT$bulk_mod <- NULL

knitr::kable(DT, align = "r", caption = "Example 21.")

# Set package options to default values
formatdown_options(reset = TRUE)

## -----------------------------------------------------------------------------
formatdown_options(size = "small")

## -----------------------------------------------------------------------------
# Use water data included with formatdown
DT <- copy(water)[1:6]

# Routine decimal formatting
cols <- c("temp", "dens", "sp_wt")
DT[, (cols) := lapply(.SD, function(x) format_dcml(x)), .SDcols = cols]

# Power of ten
DT$bulk_mod <- format_engr(DT$bulk_mod, 3)
DT$visc <- format_engr(DT$visc, 4, set_power = -3)

knitr::kable(DT, align = "r", caption = "Example 22.")

## -----------------------------------------------------------------------------
formatdown_options(reset = TRUE)

## -----------------------------------------------------------------------------
formatdown_options(size = "small")

## -----------------------------------------------------------------------------
# Use water data included with formatdown
DT <- copy(metals)

# Examine the data frame
DT[]

# Text
DT$metal <- format_text(DT$metal)

# Decimal
cols <- c("dens", "thrm_cond")
DT[, (cols) := lapply(.SD, function(x) format_dcml(x)), .SDcols = cols]

# Power of ten
cols <- c("elast_mod", "thrm_exp")
DT[, (cols) := lapply(.SD, function(x) format_engr(x, 3)), .SDcols = cols]

knitr::kable(DT, align = "lrrrr", caption = "Example 23.")

## -----------------------------------------------------------------------------
formatdown_options(reset = TRUE)

# Internal functions not exported:

# Obtain size macro
get_size_markup <- function(size) {
  # Default is no size markup
  size_markup  <- NULL
  # Allow shorthand and markup style
  if (isTRUE(!is.na(size)) & isTRUE(!is.null(size))) {
    if(size == "huge"       | size == "\\huge")       size_markup  <- "\\huge "
    if(size == "large"      | size == "\\large")      size_markup  <- "\\large "
    if(size == "normalsize" | size == "\\normalsize") size_markup  <- "\\normalsize "
    if(size == "small"      | size == "\\small")      size_markup  <- "\\small "
    if(size == "scriptsize" | size == "\\scriptsize") size_markup  <- "\\scriptsize "
  }
  return(size_markup)
}

# Determine indices to decimal (non-power) rows
assign_non_power_rows <- function(DT, format, omit_power, set_power) {

  # Indicate these are not unbound symbols (R CMD check Note)
  exponent <- NULL
  exp_raw <- NULL
  omit <- NULL
  x <- NULL

  # Create default
  DT[, omit := FALSE]

   # Cases for omit TRUE
  DT[exponent %between% omit_power, omit := TRUE]

  # recompute for engr
  if (format == "engr") {
    DT[, exponent := floor(exp_raw / 3) * 3]
    DT[exponent %between% omit_power, omit := TRUE]
  }

  # Issue #10, when x = 0 Log10(0) yields Inf
  DT[x == 0, omit := TRUE]

  # Override all if set_power has been assigned
  if (isTRUE(!is_null_or_na(set_power))) {
    DT[, exponent := set_power]
    DT[x != 0, omit := FALSE]
  }

  # Identify the non-power rows
  non_pow <- DT$omit

  return(non_pow)
}

# Prepare result with size, delim, and back to vector form
prepare_output_vector <- function(DT, size_markup, delim) {
  # Indicate these are not unbound symbols (R CMD check Note)
  value <- NULL
  # DT$value is a power-of-ten or decimal character
  # Add font size prefix
  DT[, value := paste0(size_markup, value)]
  # Surround with math delimiters (see utils.R)
  DT <- add_delim(DT, col_name = "value", delim = delim)
  # Convert to vector output
  DT <- DT[, (value)]
  return(DT)
}

get_units_suffix <- function(units_string, whitespace) {
  if (length(units_string) > 0) {
    # regex to find: optional - sign, followed by any number of digits
    u_regex <- "-?[0-9]+"
    # units exponents, if any
    u_unbraced <- regmatches(units_string, gregexec(u_regex, units_string))[[1]]
    if (length(u_unbraced) > 0) {
      # add braces and exponent symbol
      u_braced <- paste0("^{", u_unbraced, "}")
      # substitute each braced exponent for each un-braced exponent
      regmatches(units_string, gregexec(u_regex, units_string))[[1]] <- u_braced
    }
    units_string <- gsub(" ", whitespace, units_string)
  }
  return(units_string)
}

#' Format numbers
#'
#' Convert a numeric vector to a character vector in which the numbers are
#' formatted in power-of-ten notation in scientific or engineering form and
#' delimited for rendering as inline equations in an R markdown document.
#' Decimal numbers can be similarly formatted, without the power-of-ten
#' notation.
#'
#' Given a number, a numerical vector, or a numerical column from a data frame,
#' `format_numbers()` converts the numbers to character strings of the form,
#' `"$a \\times 10^{n}$"`, where `a` is the coefficient to a specified
#' number of significant digits and `n` is the exponent. When used for decimal
#' notation, `format_numbers()` converts numbers to character strings of the
#' form `"$a$"`.
#'
#' Powers-of-ten notation is omitted over a range of exponents via `omit_power`
#' such that numbers so specified are converted to decimal notation. For
#' example, the default `omit_power = c(-1, 2)` formats numbers such as 0.123,
#' 1.23, 12.3, and 123 in decimal form. To cancel these exceptions and convert
#' all numbers to powers-of-ten notation, set the `omit_power` argument to NULL
#' or NA.
#'
#' Delimiters for inline math markup can be edited if necessary. If the default
#' argument fails, try using `"\\("` as an alternative. If using a custom
#' delimiter to suit the markup environment, be sure to escape all special
#' symbols.
#'
#' When inputs are of class "units" (created with the units package), a
#' math-text macro of the form `\\mathrm{<units_string>}` is appended
#' to the formatted numerical value inside the math delimiters.
#'
#' Arguments after the dots (`...`) must be referred to by name.
#'
#' @param x              `r param_x`
#' @param digits         `r param_digits`
#' @param format         `r param_format`
#' @param ...            `r param_dots`
#' @param omit_power     `r param_omit_power`
#' @param set_power      `r param_set_power`
#' @param delim          `r param_delim`
#' @param size           `r param_size`
#' @param decimal_mark   `r param_decimal_mark`
#' @param big_mark       `r param_big_mark`
#' @param big_interval   `r param_big_interval`
#' @param small_mark     `r param_small_mark`
#' @param small_interval `r param_small_interval`
#' @param whitespace     `r param_whitespace`
#' @param multiply_mark  `r param_multiply_mark`
#'
#' @return A character vector in which numbers are formatted in power-of-ten
#' or decimal notation and delimited for rendering as inline equations
#' in an R markdown document.
#'
#' @family format_*
#' @example man/examples/examples_format_numbers.R
#' @export
format_numbers <- function(x,
                           digits = 4,
                           format = "engr",
                           ...,
                           omit_power = c(-1, 2),
                           set_power = NULL,
                           delim          = formatdown_options("delim"),
                           size           = formatdown_options("size"),
                           decimal_mark   = formatdown_options("decimal_mark"),
                           big_mark       = formatdown_options("big_mark"),
                           big_interval   = formatdown_options("big_interval"),
                           small_mark     = formatdown_options("small_mark"),
                           small_interval = formatdown_options("small_interval"),
                           whitespace     = formatdown_options("whitespace"),
                           multiply_mark  = formatdown_options("multiply_mark")) {

  # Overhead ----------------------------------------------------------------

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_numbers()")

  # Indicate these are not unbound symbols (R CMD check Note)
  size_markup <- NULL
  char_coeff <- NULL
  exponent <- NULL
  units_x <- NULL
  exp_raw <- NULL
  coeff <- NULL
  value <- NULL

  # Assign arguments -------------------------------------------------------

  # omit_power
  if (is_null_or_na(omit_power))             omit_power <- c(0.5, 0.5)
  if (is_numeric_and_length_one(omit_power)) omit_power <- rep(omit_power, 2)
  if (isTRUE(format == "dcml"))              omit_power <- c(-Inf, Inf)

  # set_power
  if (is_null_or_na(set_power) | isTRUE(format == "dcml")) set_power <- NA_real_

  # size
  if (is_null_or_na(size)) size <- NA_character_

  # grouping digits
  if (is_null_or_na(big_mark))       big_mark   <- ""
  if (is_null_or_na(big_interval))   big_mark   <- ""
  if (is_null_or_na(small_mark))     small_mark <- ""
  if (is_null_or_na(small_interval)) small_mark <- ""

  # Argument checks ---------------------------------------------------------

  # x: class "units" or "numeric" both allowed
  checkmate::assert(
    checkmate::check_class(x, "units"),
    checkmate::check_class(x, "numeric"),
    combine = "or"
  )

  # x: numeric, length at least 1
  checkmate::qassert(x, "n+")

  # digits: numeric, not missing, length 1, between 1 and 20
  checkmate::qassert(digits, "N1")
  checkmate::assert_choice(digits, choices = c(1:20))

  # format: character, not missing, length 1, element of set
  checkmate::qassert(format, "S1")
  checkmate::assert_choice(format, choices = c("engr", "sci", "dcml"))

  # omit_power: numeric, length 2, c(p, q) with p <= q
  checkmate::qassert(omit_power, "n2")
  if (!isTRUE(all(omit_power == cummax(omit_power)))) {
    stop("In `omit_power = c(p, q)`, `p` must be less than or equal to `q`.")
  }

  # set_power: numeric, length 1 (can be NA)
  checkmate::qassert(set_power, "n1")

  # Assertions on options (in utils.R)
  param_assert_delim(delim)
  param_assert_size(size)
  param_assert_mark(big_mark)
  param_assert_mark(small_mark)
  param_assert_whitespace(whitespace)

  # decimal_mark: character, not missing, length 1, period or comma
  checkmate::qassert(decimal_mark, "S1")
  checkmate::assert_choice(decimal_mark, choices = c(".", ","))

  # big_mark, small_mark

  # intervals: integer-ish, length 1 (can be NA), equal to 0,1,2,...
  checkmate::qassert(big_interval  , "x1[0,)")
  checkmate::qassert(small_interval, "x1[0,)")
  if (isTRUE(big_interval < 1))   {big_mark   = ""}
  if (isTRUE(small_interval < 1)) {small_mark = ""}

  # multiply_mark: character, not missing, length 1, two options
  checkmate::qassert(multiply_mark, "S1")
  checkmate::assert_choice(multiply_mark, choices = c("\\times", "\\cdot"))

  # Initial processing -----------------------------------------

  # Set units flag
  x_has_units <- checkmate::test_class(x, classes = c("units"))

  # store units separately and drop the units from x
  if (x_has_units) {
    units_x  <- deparse_unit(x)
    units(x) <- NULL
    # Create unit_space from whitespace by dropping the first backslash
    unit_space <- substr(whitespace, start = 2, stop = 100000L)
  }

  # Assign markup parameters
  size_markup <- get_size_markup(size)
  if(big_mark   == "thin") big_mark   <- "\\\\,"
  if(small_mark == "thin") small_mark <- "\\\\,"

  # Begin ------------------------------------------------------

  # Convert vector to data.table for processing
  DT <- data.table::copy(data.frame(x))
  data.table::setDT(DT)

  # Raw decimal exponent and integer exponent
  DT[, exp_raw := log10(abs(x))]
  DT[, exponent := floor(exp_raw)]

  # Determine power and non-power rows
  non_pow <- assign_non_power_rows(DT, format, omit_power, set_power)
  pow_10  <- !non_pow

  # Create the decimal coefficient
  DT[non_pow, coeff := x]
  DT[pow_10,  coeff := x / 10^exponent]

  # Apply signif digits to the coefficient before formatC()
  DT[, coeff := signif(coeff, digits = digits)]

  # Format the coefficient as character
  DT[, char_coeff := formatC(coeff,
                             format         = "fg",
                             digits         = digits,
                             decimal.mark   = decimal_mark,
                             big.mark       = big_mark,
                             big.interval   = big_interval,
                             small.mark     = small_mark,
                             small.interval = small_interval,
                             flag           = "#")]

  # Remove trailing decimal point and spaces created by formatC() if any
  # (see utils.R)
  DT <- omit_formatC_extras(DT, col_name = "char_coeff")

  # Construct various string values
  # Desired character output is in the value column
  DT[non_pow, value := char_coeff]
  DT[pow_10, value := paste0(char_coeff, " ", multiply_mark, " ", "10^{", exponent, "}")]
  DT[is.na(x), value := "\\mathrm{NA}"]

  # Add units suffix if any
  if (x_has_units){
    u_suffix <- get_units_suffix(units_x, whitespace)
    DT[, value := paste0(value, unit_space, "\\mathrm{", u_suffix, "}")]
  }

  # Prep for output
  output <- prepare_output_vector(DT, size_markup, delim)
  # enable printing (see data.table FAQ 2.23)
  output[]
  return(output)
}







#' Format numbers in scientific notation
#'
#' Convert a numeric vector to a character vector in which the numbers are
#' formatted in power-of-ten notation in scientific form and delimited for
#' rendering as inline equations in an R markdown document.
#'
#' `format_sci()` is a wrapper for the more general function `format_numbers()`.
#' Where defaults are defined by `formatdown_options()`, users may reassign
#' the arguments locally in the function call or globally using
#' `formatdown_options()`.
#'
#' Arguments after the dots (`...`) must be referred to by name.
#'
#' @return A character vector in which numbers are formatted in power-of-ten
#' notation in scientific form and delimited for rendering as inline equations
#' in an R markdown document.
#'
#' @example man/examples/examples_format_sci.R
#' @inherit format_numbers
#' @family format_*
#' @export
format_sci <- function(x,
                       digits = 4,
                       ...,
                       omit_power = c(-1, 2),
                       set_power = NULL,

                       # options
                       delim          = formatdown_options("delim"),
                       size           = formatdown_options("size"),
                       decimal_mark   = formatdown_options("decimal_mark"),
                       small_mark     = formatdown_options("small_mark"),
                       small_interval = formatdown_options("small_interval"),
                       whitespace     = formatdown_options("whitespace"),
                       multiply_mark  = formatdown_options("multiply_mark")) {

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_sci()")

  # wrap format_numbers()
  output <- format_numbers(x          = x,
                           digits     = digits,
                           omit_power = omit_power,
                           set_power  = set_power,

                           # options
                           delim          = delim,
                           size           = size,
                           decimal_mark   = decimal_mark,
                           small_mark     = small_mark,
                           small_interval = small_interval,
                           whitespace     = whitespace,
                           multiply_mark  = multiply_mark,

                           # wrapper pre-sets
                           format       = "sci",
                           big_mark     = formatdown_options("big_mark"),
                           big_interval = formatdown_options("big_interval")
  )

  # enable printing (see data.table FAQ 2.23)
  output[]
  return(output)
}




#' Format engineering notation
#'
#' Convert a numeric vector to a character vector in which the numbers are
#' formatted in power-of-ten notation in engineering form and delimited for
#' rendering as inline equations in an R markdown document.
#'
#' In engineering notation, all exponents are multiples of three.
#' `format_engr()` is a wrapper for the more general function `format_numbers()`.
#' Where defaults are defined by `formatdown_options()`, users may reassign
#' the arguments locally in the function call or globally using
#' `formatdown_options()`.
#'
#' Arguments after the dots (`...`) must be referred to by name.
#'
#' @return A character vector in which numbers are formatted in power-of-ten
#' notation in engineering form and delimited for rendering as inline equations
#' in an R markdown document.
#'
#' @example man/examples/examples_format_engr.R
#' @inherit format_numbers
#' @family format_*
#' @export
format_engr <- function(x,
                        digits = 4,
                        ...,
                        omit_power = c(-1, 2),
                        set_power = NULL,

                        # options
                        delim          = formatdown_options("delim"),
                        size           = formatdown_options("size"),
                        decimal_mark   = formatdown_options("decimal_mark"),
                        small_mark     = formatdown_options("small_mark"),
                        small_interval = formatdown_options("small_interval"),
                        whitespace     = formatdown_options("whitespace"),
                        multiply_mark  = formatdown_options("multiply_mark")) {

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_engr()")

  # wrap format_numbers()
  output <- format_numbers(x          = x,
                           digits     = digits,
                           omit_power = omit_power,
                           set_power  = set_power,

                           # options
                           delim          = delim,
                           size           = size,
                           decimal_mark   = decimal_mark,
                           small_mark     = small_mark,
                           small_interval = small_interval,
                           whitespace     = whitespace,
                           multiply_mark  = multiply_mark,

                           # wrapper pre-sets
                           format       = "engr",
                           big_mark     = formatdown_options("big_mark"),
                           big_interval = formatdown_options("big_interval")
  )

  # enable printing (see data.table FAQ 2.23)
  output[]
  return(output)
}






#' Format numbers in decimal notation
#'
#' Convert a numeric vector to a character vector in which the numbers are
#' formatted in decimal form and delimited for rendering as inline equations
#' in an R markdown document.
#'
#' `format_dcml()` is a wrapper for the more general function `format_numbers()`.
#' Where defaults are defined by `formatdown_options()`, users may reassign
#' the arguments locally in the function call or globally using
#' `formatdown_options()`.
#'
#' Arguments after the dots (`...`) must be referred to by name.
#'
#' @return A character vector in which numbers are formatted in decimal form
#' and delimited for rendering as inline equations in an R markdown document.
#'
#' @example man/examples/examples_format_dcml.R
#' @inherit format_numbers
#' @family format_*
#' @export
format_dcml <- function(x,
                        digits = 4,
                        ...,

                        # options
                        delim          = formatdown_options("delim"),
                        size           = formatdown_options("size"),
                        decimal_mark   = formatdown_options("decimal_mark"),
                        big_mark       = formatdown_options("big_mark"),
                        big_interval   = formatdown_options("big_interval"),
                        small_mark     = formatdown_options("small_mark"),
                        small_interval = formatdown_options("small_interval"),
                        whitespace     = formatdown_options("whitespace")) {

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_dcml()")

  # wrap for format_numbers()
  output <- format_numbers(x      = x,
                           digits = digits,

                           # options
                           delim          = delim,
                           size           = size,
                           decimal_mark   = decimal_mark,
                           big_mark       = big_mark,
                           big_interval   = big_interval,
                           small_mark     = small_mark,
                           small_interval = small_interval,
                           whitespace     = whitespace,

                           # wrapper presets
                           format     = "dcml",
                           omit_power = NULL,
                           set_power  = NULL,
                           multiply_mark = formatdown_options("multiply_mark"))

  # enable printing (see data.table FAQ 2.23)
  output[]
  return(output)
}

test_format_numbers <- function() {

  # All tests, e.g., expect_equal, expect_error, etc., are from the
  # tinytest package. Possible test functions listed at
  # https://github.com/markvanderloo/tinytest/blob/master/pkg/README.md

  # usage
  # format_numbers <- function(x,
  #                            digits = 4,
  #                            format = "engr",
  #                            ...,
  #                            omit_power = c(-1, 2),
  #                            set_power = NULL,
  #                            delim          = formatdown_options("delim"),
  #                            size           = formatdown_options("size"),
  #                            decimal_mark   = formatdown_options("decimal_mark"),
  #                            big_mark       = formatdown_options("big_mark"),
  #                            big_interval   = formatdown_options("big_interval"),
  #                            small_mark     = formatdown_options("small_mark"),
  #                            small_interval = formatdown_options("small_interval"),
  #                            whitespace     = formatdown_options("whitespace"))

  # Needed for tinytest::build_install_test()
  suppressPackageStartupMessages(library(data.table))

  # Inputs: First three arguments and convenience functions
  # +/- exponents and default digits and format
  x <- 6.0221E+23
  ans101 <- "$602 \\times 10^{21}$"
  expect_equal(ans101, format_numbers(x = x, digits = 3, format = "engr"))
  expect_equal(ans101, format_numbers(x, 3, "engr"))
  expect_equal(ans101, format_engr(x, 3))

  x <- 103219
  ans102 <- "$103200$"
  expect_equal(ans102, format_numbers(x, format = "dcml"))
  expect_equal(ans102, format_dcml(x, 4))
  expect_equal(ans102, format_dcml(x))

  x <- 6.0221E-23
  ans103 <- "$6.0221 \\times 10^{-23}$"
  expect_equal(ans103, format_numbers(x, 5, "sci"))
  expect_equal(ans103, format_sci(x, 5))

  # Input of class units
  x <- 6.0221E+23
  units(x) <- "N m2 C-2"
  ans104 <- "$602.2 \\times 10^{21}\\ \\mathrm{m^{2}\\ N\\ C^{-2}}$"
  ans105 <- "$602.2 \\times 10^{21}\\ \\mathrm{N\\ m^{2}\\ C^{-2}}$"
  expect_true(format_numbers(x) %in% c(ans104, ans105))

  units(x) <- NULL
  expect_equal(ans101, format_numbers(x, 3))

  # Large and small values format correctly
  x <- 2.2e-16
  ans106 <- "$2.2 \\times 10^{-16}$"
  expect_equal(ans106, format_sci(x, 2))

  x <- 1.7e+300
  ans107 <- "$1.7 \\times 10^{300}$"
  expect_equal(ans107, format_sci(x, 2))

  # Inputs: Data frame,one column as vector
  x <- air_meas[, (pres)]
  ans108 <- c("$101.1 \\times 10^{3}$",
             "$101.0 \\times 10^{3}$",
             "$101.1 \\times 10^{3}$",
             "$101.0 \\times 10^{3}$",
             "$101.1 \\times 10^{3}$")
  expect_equal(ans108, format_numbers(x))

  # Inputs: Data frame, selected column in place
  DT  <- air_meas[, .(trial, pres)]
  ans109 <- data.table(wrapr::build_frame(
    "trial"  , "pres"               |
      "a"    , "$101.1 \\times 10^{3}$" |
      "b"    , "$101.0 \\times 10^{3}$" |
      "c"    , "$101.1 \\times 10^{3}$" |
      "d"    , "$101.0 \\times 10^{3}$" |
      "e"    , "$101.1 \\times 10^{3}$" ))
  cols_we_want <- c("pres")
  DT <- DT[, (cols_we_want) := lapply(.SD, function(x) format_numbers(x)),
           .SDcols = cols_we_want]
  expect_equal(ans109, DT)

  # Negative values OK
  x <- -6.3212e+12
  ans110 <- "$-6.321 \\times 10^{12}$"
  expect_equal(ans110, format_numbers(x))

  # Spaces inserted by formatC are trimmed. In this test, the first number
  # has four characters (the decimal point is a character); formatC() adds
  # spaces to store "800" as " 800". format_numbers() removes the extra spaces
  # NOTE: Can change to format() from formatC() to eliminate this issue.
  x <- c(1.02, 0.8) * 1.0e-6
  ans111 <- c("$1.02 \\times 10^{-6}$", "$800 \\times 10^{-9}$")
  expect_equal(ans111, format_numbers(x, 3))

  # Digits
  x <- 9.75358e+5
  ans151 <- "$9.8 \\times 10^{5}$"
  expect_equal(ans151, format_sci(x, 2))
  ans152 <- "$9.75 \\times 10^{5}$"
  expect_equal(ans152, format_sci(x, 3))
  ans153 <- "$9.754 \\times 10^{5}$"
  expect_equal(ans153, format_sci(x, 4))

  # Combinations of format, set_power, and omit_power
  x <- 0.00021541
  ans201 <- "$2.154 \\times 10^{-4}$"
  expect_equal(ans201, format_sci(x, omit_power = NULL))
  expect_equal(ans201, format_sci(x, omit_power = -3))

  x <- 3.2211e-3
  ans202 <- "$0.003221$"
  expect_equal(ans202, format_sci(x, omit_power = -3))
  expect_equal(ans202, format_engr(x, omit_power = -3))
  # expect_equal(ans202, format_dcml(x, omit_power = -3))
  expect_equal(ans202, format_dcml(x))

  x <- 5.222
  ans203 <- "$5.222 \\times 10^{0}$"
  expect_equal(ans203, format_sci(x, omit_power = NULL))
  expect_equal(ans203, format_engr(x, omit_power = NULL))

  ans204 <- "$5.222$"
  expect_equal(ans204, format_sci(x, omit_power = c(-1, 0)))
  expect_equal(ans204, format_engr(x, omit_power = c(-1, 0)))

  x <- 22.11
  ans205 <- "$2.211 \\times 10^{1}$"
  expect_equal(ans205, format_sci(x, omit_power = NULL))
  expect_equal(ans205, format_sci(x, omit_power = c(-1, 0)))

  ans206 <- "$22.11 \\times 10^{0}$"
  expect_equal(ans206, format_engr(x, omit_power = NULL))
  ans207 <- "$22.11$"
  expect_equal(ans207, format_engr(x, omit_power = c(-1, 0)))

  # omit_power has no effect
  x <- 6.521
  ans208 <- "$6.521 \\times 10^{0}$"
  expect_equal(ans208, format_sci(x, omit_power = NULL))
  expect_equal(ans208, format_sci(x, omit_power = -3))
  expect_equal(ans208, format_engr(x, omit_power = NULL))
  expect_equal(ans208, format_engr(x, omit_power = -3))

  # set_power overrides omit_power
  x <- 101300
  ans209 <- format_sci(x, set_power = 2)
  expect_equal(ans209, format_sci(x, omit_power = 5, set_power = 2))
  expect_equal(ans209, format_sci(x, omit_power = 2, set_power = 2))

  # set_power overrides format
  x   <- 6.0221E+23
  ans209b <- "$0.6022 \\times 10^{24}$"
  expect_equal(ans209b, format_engr(x, set_power = 24L))
  expect_equal(ans209b, format_sci(x, set_power = 24L))

  # dcml ignores set_power; omit_power creates a decimal
  x <- 101300
  ans210 <- format_dcml(x)
  # expect_equal(ans210, format_dcml(x, set_power = 2))
  expect_equal(ans210, format_engr(x, omit_power = 5))

  # Data frame,one column as vector
  x   <- air_meas[, (pres)]
  ans15 <- c("$101.1 \\times 10^{3}$",
             "$101.0 \\times 10^{3}$",
             "$101.1 \\times 10^{3}$",
             "$101.0 \\times 10^{3}$",
             "$101.1 \\times 10^{3}$")
  expect_equal(format_numbers(x), ans15)

  # Data frame, selected column in place
  DT  <- air_meas[, .(trial, pres)]
  ans16 <- data.table(wrapr::build_frame(
    "trial"  , "pres"               |
      "a"    , "$101.1 \\times 10^{3}$" |
      "b"    , "$101.0 \\times 10^{3}$" |
      "c"    , "$101.1 \\times 10^{3}$" |
      "d"    , "$101.0 \\times 10^{3}$" |
      "e"    , "$101.1 \\times 10^{3}$" ))
  cols_we_want <- c("pres")
  DT <- DT[, (cols_we_want) := lapply(.SD, function(x) format_numbers(x)),
           .SDcols = cols_we_want]
  expect_equal(DT, ans16)

  # ERRORS  ------------------------
  x <- 101300

  # arguments of incorrect class
  expect_error(format_numbers(x = TRUE))
  expect_error(format_numbers(x, digits = TRUE))
  expect_error(format_sci(x, omit_power = TRUE))
  expect_error(format_sci(x, set_power = TRUE))

  # arguments not among allowed choices
  expect_error(format_numbers(x = NULL))
  expect_error(format_numbers(x, digits = NULL))
  expect_error(format_numbers(x, digits = 22))
  expect_error(format_numbers(x, format = NULL))
  expect_error(format_numbers(x, format = "scientific"))

  # arguments with incorrect relationship
  expect_error(format_sci(x, omit_power = c(3, -1)))

  # arguments of incorrect length
  expect_error(format_numbers(x, digits = c(1, 2)))
  expect_error(format_numbers(x, format = c("engr", "sci")))
  expect_error(format_numbers(x, omit_power = c(-1, 0, 3)))
  expect_error(format_numbers(x, set_power = c(-1, 2)))

  # arguments after dots must be named
  expect_error(format_numbers(x, 4, "engr", c(-1, 2)))
  expect_error(format_sci(x, 4, c(-1, 2)))
  expect_error(format_engr(x, 4, c(-1, 2)))
  expect_error(format_dcml(x, 4, "$"))

  # function output not printed
  invisible(NULL)
}

test_format_numbers()

test_delim <- function() {

  # Option: delim
  #         format_numbers()
  #         format_sci()
  #         format_engr()
  #         format_dcml()
  #         format_text()

  # Possible values are
  #         "$"
  #         c("$", "$")
  #         "\("
  #         c("\\(", "\\)")
  #         c(<left_marker>, <right_marker>) user-defined

  # correct behaviors
  x <- 6.0221e+23

  ans101 <- "$602.2 \\times 10^{21}$"
  expect_equal(ans101, format_numbers(x))
  expect_equal(ans101, format_numbers(x, delim = "$"))
  expect_equal(ans101, format_numbers(x, delim = c("$", "$")))
  expect_equal(ans101, format_engr(x))
  expect_equal(ans101, format_engr(x, delim = "$"))
  expect_equal(ans101, format_engr(x, delim = c("$", "$")))

  ans102 <- "\\(602.2 \\times 10^{21}\\)"
  expect_equal(ans102, format_numbers(x, delim = "\\("))
  expect_equal(ans102, format_numbers(x, delim = c("\\(", "\\)")))
  expect_equal(ans102, format_engr(x, delim = "\\("))
  expect_equal(ans102, format_engr(x, delim = c("\\(", "\\)")))

  ans103 <- "\\[602.2 \\times 10^{21}\\]"
  expect_equal(ans103, format_numbers(x, delim = c("\\[", "\\]")))
  expect_equal(ans103, format_engr(x, delim = c("\\[", "\\]")))

  ans104 <- "$6.022 \\times 10^{23}$"
  expect_equal(ans104, format_sci(x))
  expect_equal(ans104, format_sci(x, delim = "$"))
  expect_equal(ans104, format_sci(x, delim = c("$", "$")))

  ans105 <- "\\(6.022 \\times 10^{23}\\)"
  expect_equal(ans105, format_sci(x, delim = "\\("))
  expect_equal(ans105, format_sci(x, delim = c("\\(", "\\)")))

  ans106 <- "\\[6.022 \\times 10^{23}\\]"
  expect_equal(ans106, format_sci(x, delim = c("\\[", "\\]")))

  x <- 987123

  ans107 <- "$987100$"
  expect_equal(ans107, format_dcml(x))
  expect_equal(ans107, format_dcml(x, delim = "$"))
  expect_equal(ans107, format_dcml(x, delim = c("$", "$")))

  ans108 <- "\\(987100\\)"
  expect_equal(ans108, format_dcml(x, delim = "\\("))
  expect_equal(ans108, format_dcml(x, delim = c("\\(", "\\)")))

  ans109 <- "\\[987100\\]"
  expect_equal(ans109, format_dcml(x, delim = c("\\[", "\\]")))

  x <- 6.0221e+23
  units(x) <- "m"

  ans110 <- "$602.2 \\times 10^{21}\\ \\mathrm{m}$"
  expect_equal(ans110, format_numbers(x))
  expect_equal(ans110, format_numbers(x, delim = "$"))
  expect_equal(ans110, format_numbers(x, delim = c("$", "$")))

  ans111 <- "\\(602.2 \\times 10^{21}\\ \\mathrm{m}\\)"
  expect_equal(ans111, format_numbers(x, delim = "\\("))
  expect_equal(ans111, format_numbers(x, delim = c("\\(", "\\)")))

  ans112 <- "\\[602.2 \\times 10^{21}\\ \\mathrm{m}\\]"
  expect_equal(ans112, format_numbers(x, delim = c("\\[", "\\]")))

  x <- "hello world"

  ans113 <- "$\\mathrm{hello\\ world}$"
  expect_equal(ans113, format_text(x))
  expect_equal(ans113, format_text(x, delim = "$"))
  expect_equal(ans113, format_text(x, delim = c("$", "$")))

  ans114 <- "\\(\\mathrm{hello\\ world}\\)"
  expect_equal(ans114, format_text(x, delim = "\\("))
  expect_equal(ans114, format_text(x, delim = c("\\(", "\\)")))

  ans115 <- "\\[\\mathrm{hello\\ world}\\]"
  expect_equal(ans115, format_text(x, delim = c("\\[", "\\]")))

  # vector input
  x <- c(123, 4.56e+11)
  ans116 <- c("\\(123\\)", "\\(456 \\times 10^{9}\\)")
  expect_equal(ans116, format_numbers(x, 3, delim = "\\("))

  # errors
  expect_error(format_numbers(x, delim = c("\\[")))
  expect_error(format_numbers(x, delim = c(NA_character_, NA_character_)))
  expect_error(format_numbers(x, delim = c(NULL, NULL)))

  # function output not printed
  invisible(NULL)
}

test_delim()

test_formatdown_options <- function() {

  # All tests, e.g., expect_equal, expect_error, etc., are from the
  # tinytest package. Possible test functions listed at
  # https://github.com/markvanderloo/tinytest/blob/master/pkg/README.md

  # defaults              possibles
  # delim = "$".          "\(" or c("$", "$") or c("\\(", "\\)")
  # size = NULL.          "scriptsize", "small", "normalsize", "large", and "huge"
  # decimal_mark = "."    ","
  # big_mark = ""         "thin" or "\\\\,"
  # big_interval = 3      any positive integer
  # small_mark = ""       "thin" or "\\\\,"
  # small_interval = 5    any positive integer
  # whitespace = "\\\\>"  "\\\\:" or "⁠\\\\ ⁠".,
  # Z = FALSE,
  # warn = TRUE

  # Store existing settings
  prior_settings <- formatdown_options()

  # Reset to default settings
  formatdown_options(reset = TRUE)

  x <- 6.0221e+23

  # delim
  ans901 <- "$"
  expect_equal(ans901, formatdown_options("delim"))

  formatdown_options(delim = "\\(")
  ans902 <- "\\(602.2 \\times 10^{21}\\)"
  expect_equal(ans902, format_numbers(x))
  formatdown_options(reset = TRUE)

  # size
  ans903 <- NULL
  expect_equal(ans903, formatdown_options("size"))

  formatdown_options(size = "small")
  ans904 <- "$\\small 602.2 \\times 10^{21}$"
  expect_equal(ans904, format_numbers(x))
  formatdown_options(reset = TRUE)

  # decimal_mark
  ans905 <- "."
  expect_equal(ans905, formatdown_options("decimal_mark"))

  formatdown_options(decimal_mark = ",")
  ans906 <- "$602,2 \\times 10^{21}$"
  expect_equal(ans906, format_numbers(x))
  formatdown_options(reset = TRUE)

  # big_mark
  x <- 123400
  ans907 <- ""
  expect_equal(ans907, formatdown_options("big_mark"))

  formatdown_options(big_mark = "thin")
  ans908 <- "$123\\,400$"
  expect_equal(ans908, format_dcml(x))
  formatdown_options(reset = TRUE)

  # big_interval
  ans909 <- 3
  expect_equal(ans909, formatdown_options("big_interval"))

  formatdown_options(big_interval = 4)
  ans910 <- "$12\\,3400$"
  expect_equal(ans910, format_dcml(x, big_mark = "thin"))
  formatdown_options(reset = TRUE)

  # small_mark
  x <- 0.9876543211
  ans911 <- ""
  expect_equal(ans911, formatdown_options("small_mark"))

  formatdown_options(small_mark = "thin")
  ans912 <- "$0.98765\\,43211$"
  expect_equal(ans912, format_dcml(x, 10))
  formatdown_options(reset = TRUE)

  # small_interval
  ans913 <- 5
  expect_equal(ans913, formatdown_options("small_interval"))

  formatdown_options(small_interval = 4)
  ans914 <- "$0.9876\\,5432\\,11$"
  expect_equal(ans914, format_dcml(x, 10, small_mark = "thin"))
  formatdown_options(reset = TRUE)

  # whitespace
  ans915 <- "\\\\ "
  expect_equal(ans915, formatdown_options("whitespace"))

  x <- "hello world"
  formatdown_options(whitespace = "\\\\:")
  ans916 <- "$\\mathrm{hello\\:world}$"
  expect_equal(ans916, format_text(x))

  x <- pi
  units(x) <- "m s-2"
  ans917 <- "$3.142\\:\\mathrm{m\\:s^{-2}}$"
  expect_equal(ans917, format_dcml(x))
  formatdown_options(reset = TRUE)

  # set multiple options
  x <- 0.9876543211
  formatdown_options(decimal_mark = ",", small_mark = "thin", small_interval = 4)
  ans918 <- "$0,9876\\,5432\\,11$"
  expect_equal(ans918, format_dcml(x, 10))
  formatdown_options(reset = TRUE)

  # multiply_mark
  x <- 6.0221e+23
  formatdown_options(decimal_mark = ",", multiply_mark = "\\cdot")
  ans919 <- "$6,022 \\cdot 10^{23}$"
  expect_equal(ans919, format_sci(x))
  formatdown_options(reset = TRUE)

  # Z (atomic number)
  ans920 <- FALSE
  expect_equal(ans920, formatdown_options("Z"))

  formatdown_options("Z" = TRUE)

  ans921 <- TRUE
  expect_equal(ans921, formatdown_options("Z"))

  formatdown_options(Z = FALSE)


  # isotope warning
  ans922 <- TRUE
  expect_equal(ans922, formatdown_options("warn"))

  formatdown_options("warn" = FALSE)

  ans923 <- FALSE
  expect_equal(ans923, formatdown_options("warn"))

  formatdown_options("warn" = TRUE)



  # ERRORS  ------------------------
  expect_error(formatdown_options("x"))
  expect_error(formatdown_options("digits"))
  expect_error(formatdown_options("format"))
  expect_error(formatdown_options("omit_power"))
  expect_error(formatdown_options("set_power"))
  expect_error(formatdown_options("face"))
  formatdown_options(reset = TRUE)

  # reset options to those before this test
  do.call(formatdown_options, prior_settings)

  # confirm that rest-settings equal prior-settings
  do_call_settings <- formatdown_options()
  expect_equal(do_call_settings, prior_settings)



  # function output not printed
  invisible(NULL)
}

test_formatdown_options()

test_small_mark <- function() {

  # Options: small_mark and small_interval
  #         format_dcml()

  # Possible values are:   default ,  or:
  #         small_mark            "" , "thin" , "\\\\,"
  #         small_interval         5 ,  any positive integer

  # correct behaviors
  x <- 0.9876543211

  # small_mark
  ans601 <- "$0.9877$"
  expect_equal(ans601, format_dcml(x))
  expect_equal(ans601, format_dcml(x, small_mark = ""))
  expect_equal(ans601, format_dcml(x, small_mark = NULL))
  expect_equal(ans601, format_dcml(x, small_mark = NA))

  ans602 <- "$0.98765\\,43211$"
  expect_equal(ans602, format_dcml(x, 10, small_mark = "thin"))
  expect_equal(ans602, format_dcml(x, 10, small_mark = "\\\\,"))

  # small_interval
  ans603 <- "$0.98765\\,43211$"
  expect_equal(ans603, format_dcml(x, 10, small_mark = "thin"))
  expect_equal(ans603, format_dcml(x, 10, small_mark = "thin", small_interval = 5))
  expect_equal(ans603, format_dcml(x, 10, small_mark = "thin", small_interval = 5L))

  ans604 <- "$0.987\\,654\\,321\\,1$"
  expect_equal(ans604, format_dcml(x, 10, small_mark = "thin", small_interval = 3))

  # interactions
  # empty mark ignores interval
  ans605 <- "$0.9877$"
  expect_equal(ans605, format_dcml(x, small_mark = "", small_interval = 3))
  # zero interval overrides mark
  expect_equal(ans605, format_dcml(x, small_mark = "thin", small_interval = 0))
  # mark ignored if interval > no. of digits
  expect_equal(ans605, format_dcml(x, small_mark = "thin", small_interval = 5))

  # vector input
  x <- c(1.9876, 0.54321)
  ans606 <- c("$1.987\\,6$", "$0.543\\,21$")
  expect_equal(ans606, format_dcml(x, 5, small_mark = "thin", small_interval = 3))

  # errors
  expect_error(format_dcml(x, small_mark = ","))
  expect_error(format_dcml(x, small_mark = 0))
  expect_error(format_dcml(x, small_mark = "\\,"))
  expect_error(format_dcml(x, small_mark = "thin", small_interval = 1.5))
  expect_error(format_dcml(x, small_mark = "thin", small_interval = -1))
  expect_error(format_dcml(x, small_mark = "thin", small_interval = Inf))
  expect_error(format_dcml(x, small_mark = "thin", small_interval = NA))
  expect_error(format_dcml(x, small_mark = "thin", small_interval = NULL))

  # function output not printed
  invisible(NULL)
}

test_small_mark()

test_multiply_mark <- function() {

  # Option: multiply_mark
  #         format_numbers(..., format = c("sci", "engr"))
  #         format_sci()
  #         format_engr()

  # Possible values are
  #         "\\times"
  #         "\\cdot""

  x <- 6.0221e+23
  ans101 <- "$6.022 \\times 10^{23}$"
  expect_equal(ans101, format_sci(x))
  expect_equal(ans101, format_sci(x, multiply_mark = "\\times"))

  ans102 <- "$6.022 \\cdot 10^{23}$"
  expect_equal(ans102, format_sci(x, multiply_mark = "\\cdot"))

  # vector input
  x <- c(1.2, 103400)
  ans103 <- c("$1.200$", "$103.4 \\times 10^{3}$")
  expect_equal(ans103, format_engr(x, decimal_mark = ".", multiply_mark = "\\times"))
  ans104 <- c("$1,200$", "$103,4 \\cdot 10^{3}$")
  expect_equal(ans104, format_engr(x, decimal_mark = ",", multiply_mark = "\\cdot"))

  # errors
  expect_error(format_sci(x, multiply_mark = NA))
  expect_error(format_sci(x, multiply_mark = NULL))
  expect_error(format_sci(x, multiply_mark = 0))
  expect_error(format_sci(x, multiply_mark = TRUE))

  # function output not printed
  invisible(NULL)
}

test_multiply_mark()

test_big_mark <- function() {

  # Options: big_mark and big_interval
  #         format_dcml()
  #         format_numbers(..., format = "dcml")

  # Possible values are:   default ,  or:
  #         big_mark            "" , "thin" , "\\\\,"
  #         big_interval         3 ,  any positive integer

  # correct behaviors
  x <- 123400

  # big_mark
  ans401 <- "$123400$"
  expect_equal(ans401, format_dcml(x))
  expect_equal(ans401, format_dcml(x, big_mark = ""))
  expect_equal(ans401, format_dcml(x, big_mark = NULL))
  expect_equal(ans401, format_dcml(x, big_mark = NA))

  ans402 <- "$123\\,400$"
  expect_equal(ans402, format_dcml(x, big_mark = "thin"))
  expect_equal(ans402, format_dcml(x, big_mark = "\\\\,"))

  # big_interval
  ans403 <- "$123\\,400$"
  expect_equal(ans403, format_dcml(x, big_mark = "thin"))
  expect_equal(ans403, format_dcml(x, big_mark = "thin", big_interval = 3))
  expect_equal(ans403, format_dcml(x, big_mark = "thin", big_interval = 3L))

  ans404 <- "$12\\,3400$"
  expect_equal(ans404, format_dcml(x, big_mark = "thin", big_interval = 4))

  # interactions
  # empty mark ignores interval
  ans405 <- "$123400$"
  expect_equal(ans405, format_dcml(x, big_mark = "", big_interval = 3))
  # zero interval overrides mark
  expect_equal(ans405, format_dcml(x, big_mark = "thin", big_interval = 0))
  # mark ignored if interval > no. of digits
  expect_equal(ans405, format_dcml(x, big_mark = "thin", big_interval = 6))

  # vector input
  x <- c(123456, 789012)
  ans406 <- c("$12\\,34\\,60$", "$78\\,90\\,10$")
  expect_equal(ans406, format_dcml(x, 5, big_mark = "thin", big_interval = 2))

  # errors
  x <- 123400
  expect_error(format_dcml(x, big_mark = ","))
  expect_error(format_dcml(x, big_mark = 0))
  expect_error(format_dcml(x, big_mark = "\\,"))

  expect_error(format_dcml(x, big_mark = "thin", big_interval = 1.5))
  expect_error(format_dcml(x, big_mark = "thin", big_interval = -1))
  expect_error(format_dcml(x, big_mark = "thin", big_interval = Inf))

  expect_error(format_dcml(x, big_mark = "thin", big_interval = NA))
  expect_error(format_dcml(x, big_mark = "thin", big_interval = NULL))

  # function output not printed
  invisible(NULL)
}

test_big_mark()

test_decimal_mark <- function() {

  # Option: decimal_mark
  #         format_numbers()
  #         format_sci()
  #         format_engr()
  #         format_dcml()

  # Possible values are
  #         "."
  #         ","

  # correct behaviors
  x <- 6.0221e+23

  ans101 <- "$602.2 \\times 10^{21}$"
  expect_equal(ans101, format_numbers(x))
  expect_equal(ans101, format_numbers(x, decimal_mark = "."))

  ans102 <- "$602,2 \\times 10^{21}$"
  expect_equal(ans102, format_numbers(x, decimal_mark = ","))

  x <- pi

  ans103 <- "$3.142$"
  expect_equal(ans103, format_numbers(x))
  expect_equal(ans103, format_numbers(x, decimal_mark = "."))
  expect_equal(ans103, format_engr(x   , decimal_mark = "."))
  expect_equal(ans103, format_sci(x    , decimal_mark = "."))
  expect_equal(ans103, format_dcml(x   , decimal_mark = "."))

  ans104 <- "$3,142$"
  expect_equal(ans104, format_numbers(x, decimal_mark = ","))
  expect_equal(ans104, format_engr(x   , decimal_mark = ","))
  expect_equal(ans104, format_sci(x    , decimal_mark = ","))
  expect_equal(ans104, format_dcml(x   , decimal_mark = ","))

  # vector input
  x <- c(pi, pi*1e+11)
  ans105 <- c("$3,142$", "$314,2 \\times 10^{9}$")
  expect_equal(ans105, format_numbers(x, decimal_mark = ","))

  # errors

  expect_error(format_numbers(pi, decimal_mark = ""))
  expect_error(format_numbers(pi, decimal_mark = " "))
  expect_error(format_numbers(pi, decimal_mark = NULL))
  expect_error(format_numbers(pi, decimal_mark = NA))
  expect_error(format_numbers(pi, decimal_mark = "\\,"))

  # function output not printed
  invisible(NULL)
}

test_decimal_mark()

test_size <- function() {

  # Option: size
  #         format_numbers()
  #         format_sci()
  #         format_engr()
  #         format_dcml()
  #         format_text()

  # Possible values are
  #         NA_character_
  #         "scriptsize"
  #         "small"
  #         "normalsize"
  #         "large"
  #         "huge"

  # correct behaviors
  x <- 123
  ans101 <- "$123$"
  expect_equal(ans101, format_numbers(x, 3))
  expect_equal(ans101, format_numbers(x, 3, size = NULL))
  expect_equal(ans101, format_numbers(x, 3, size = NA_character_))

  ans102 <- "$\\scriptsize 123$"
  expect_equal(ans102, format_numbers(x, 3, size = "scriptsize"))
  expect_equal(ans102, format_sci(x, 3, size = "scriptsize"))
  expect_equal(ans102, format_engr(x, 3, size = "scriptsize"))
  expect_equal(ans102, format_dcml(x, 3, size = "scriptsize"))

  ans103 <- "$\\small 123$"
  expect_equal(ans103, format_numbers(x, 3, size = "small"))
  expect_equal(ans103, format_sci(x, 3, size = "small"))
  expect_equal(ans103, format_engr(x, 3, size = "small"))
  expect_equal(ans103, format_dcml(x, 3, size = "small"))

  ans104 <- "$\\normalsize 123$"
  expect_equal(ans104, format_numbers(x, 3, size = "normalsize"))
  expect_equal(ans104, format_sci(x, 3, size = "normalsize"))
  expect_equal(ans104, format_engr(x, 3, size = "normalsize"))
  expect_equal(ans104, format_dcml(x, 3, size = "normalsize"))

  ans105 <- "$\\large 123$"
  expect_equal(ans105, format_numbers(x, 3, size = "large"))
  expect_equal(ans105, format_sci(x, 3, size = "large"))
  expect_equal(ans105, format_engr(x, 3, size = "large"))
  expect_equal(ans105, format_dcml(x, 3, size = "large"))

  ans106 <- "$\\huge 123$"
  expect_equal(ans106, format_numbers(x, 3, size = "huge"))
  expect_equal(ans106, format_sci(x, 3, size = "huge"))
  expect_equal(ans106, format_engr(x, 3, size = "huge"))
  expect_equal(ans106, format_dcml(x, 3, size = "huge"))

  x <- "hello world"

  ans107 <- "$\\scriptsize \\mathrm{hello\\ world}$"
  expect_equal(ans107, format_text(x, size = "scriptsize"))

  ans108 <- "$\\small \\mathrm{hello\\ world}$"
  expect_equal(ans108, format_text(x, size = "small"))

  ans109 <- "$\\normalsize \\mathrm{hello\\ world}$"
  expect_equal(ans109, format_text(x, size = "normalsize"))

  ans110 <- "$\\large \\mathrm{hello\\ world}$"
  expect_equal(ans110, format_text(x, size = "large"))

  ans111 <- "$\\huge \\mathrm{hello\\ world}$"
  expect_equal(ans111, format_text(x, size = "huge"))

  # vector input
  x <- c(123, 4.56e+11)
  ans112 <- c("$\\small 123$", "$\\small 456 \\times 10^{9}$")
  expect_equal(ans112, format_numbers(x, 3, size = "small"))

  # errors
  expect_error(format_numbers(11, size = ""))
  expect_error(format_numbers(11, size = "LARGE"))
  expect_error(format_text("A"  , size = ""))

  # function output not printed
  invisible(NULL)
}

test_size()

test_whitespace <- function() {

  # Option: whitespace
  #         format_numbers() with input class units
  #         format_sci()
  #         format_engr()
  #         format_dcml()
  #         format_text()

  # Possible values are
  #         "\\\\>"
  #         "\\\\:"
  #         "⁠\\\\ ⁠"

  # correct behaviors
  x <- "hello world"

  ans701 <- "$\\mathrm{hello\\ world}$"
  expect_equal(ans701, format_text(x))
  expect_equal(ans701, format_text(x, whitespace = "\\\\ "))

  ans702 <- "$\\mathrm{hello\\:world}$"
  expect_equal(ans702, format_text(x, whitespace = "\\\\:"))

  ans703 <- "$\\mathrm{hello\\>world}$"
  expect_equal(ans703, format_text(x, whitespace = "\\\\>"))

  # numeric input with units
  x <- pi
  units(x) <- "rad/s"

  ans704 <- "$3.142\\ \\mathrm{rad\\ s^{-1}}$"
  expect_equal(ans704, format_dcml(x))

  ans705 <- "$3.142\\:\\mathrm{rad\\:s^{-1}}$"
  expect_equal(ans705, format_dcml(x, whitespace = "\\\\:"))

  ans706 <- "$3.142\\ \\mathrm{rad\\ s^{-1}}$"
  expect_equal(ans706, format_dcml(x, whitespace = "\\\\ "))

  # vector input
  x <- c("hello world", "goodbye farm")
  ans707 <- c("$\\mathrm{hello\\ world}$", "$\\mathrm{goodbye\\ farm}$")
  expect_equal(ans707, format_text(x, whitespace = "\\\\ "))

  # errors
  expect_error(format_text(x, whitespace = NA))
  expect_error(format_text(x, whitespace = NULL))
  expect_error(format_text(x, whitespace = 0))
  expect_error(format_text(x, whitespace = TRUE))

  # function output not printed
  invisible(NULL)
}

test_whitespace()

# input: single number
x <- 103400
format_dcml(x)

# input: units class
x <- 103400
units(x) <- "N m2 C-2"
format_dcml(x)

# input: vector
data("metals", package = "formatdown")
x <- metals$thrm_cond
format_dcml(x)

# significant digits
x <- 155.77
format_dcml(x, 2)
format_dcml(x, 3)
format_dcml(x, 4)

# input: data frame
x <- metals[, c("dens", "thrm_cond")]
as.data.frame(apply(x, 2, format_dcml, digits = 3))

# format_dcml() same as format_numbers(..., format = "dcml")
x <- 103400
format_dcml(x)
format_numbers(x, format = "dcml")

#' Remove trailing decimal point and spaces, if any, added by formatC()
#'
#' @param DT data.table
#' @param col_name Character, length 1, column name to work on
#' @noRd
omit_formatC_extras <- function(DT, col_name) {

  # Logical. Identify rows with trailing decimal points
  rows_we_want <- DT[, get(col_name)] %like% "[:.:]$"

  # Delete trailing decimal points if any
  DT[rows_we_want, (col_name) := sub("[:.:]", "", get(col_name))]

  # Trim space added by formatC if any
  DT[, (col_name) := trimws(get(col_name), which = "l")]
  return(DT)
}

#' Surround elements of character vector with inline math delimiters
#'
#' @param DT data.table
#' @param col_name Character, length 1, column name to work on
#' @param delim Character, delimiter, length 1 or 2
#' @noRd
add_delim <- function(DT, col_name, delim) {
  if (length(delim) == 1) {
    if (delim == "\\(" | delim == "\\)") {
      DT[, (col_name) := paste0("\\(", get(col_name), "\\)")]
    } else {
      DT[, (col_name) := paste0(delim, get(col_name), delim)]
    }
  } else {
    DT[, (col_name) := paste0(delim[1], get(col_name), delim[2])]
  }
}

#' Is an element of x NA or NULL, true/false
#'
#' @param x Vector
#' @noRd
is_null_or_na <- function(x) {
  sum(c(is.null(x), is.na(x))) > 0
}

#' Is x numeric and length 1?
#'
#' @param x Vector
#' @noRd
is_numeric_and_length_one <- function (x) {
  isTRUE(is.numeric(x) & length(x) == 1)
}

#' Param check delim
#'
#' Character, length 1 or two, not empty, not missing
#' @param delim Character
#' @noRd
param_assert_delim <- function(delim) {
  checkmate::qassert(delim, "S+")
  checkmate::assert_true(!"" %in% delim)
  checkmate::assert_true(length(delim) <= 2)
  if (length(delim) < 2) {
    checkmate::assert_choice(delim, choices = c("$", "\\("))
  }
}

#' Param check size
#'
#' @param size Character, not missing, length 1, element of set
#' @noRd
param_assert_size <- function(size) {
  checkmate::qassert(size, "s1")
  checkmate::assert_choice(
    size,
    choices = c(NA_character_, "scriptsize", "small", "normalsize", "large", "huge", "\\scriptsize", "\\small", "\\normalsize", "\\large", "\\huge")
  )
}

#' Param check whitespace
#'
#' @param whitespace Character, not missing, length 1
#' @noRd
param_assert_whitespace <- function(whitespace) {
  checkmate::qassert(whitespace, "S1")
}

#' Param check mark
#'
#' For big_mark, small_mark
#' @param mark Character, length 1, can be empty.
#' @noRd
param_assert_mark <- function(mark) {
  checkmate::qassert(mark, "s1")
  checkmate::assert_choice(mark, choices = c("thin", "\\\\,", "")
  )
}

# Text vector

# default face = "plain"
x <- air_meas$humid
format_text(x)

# equivalently
format_text(x, face = "plain")

# input vector
x <- c("Hello world!", "Goodbye blues!")
format_text(x)

# argument coerced to character string if possible
format_text(c(1.2, 2.3, 3.4))
format_text(x = NA)
format_text(x = c(TRUE, FALSE, TRUE))

# numbers as strings are rendered as-is
format_text(x = c("1.2E-3", "3.4E+0", "5.6E+3"))

# other font faces
format_text(x, face = "italic")
format_text(x, face = "bold")
format_text(x, face = "sans")
format_text(x, face = "mono")

#' @keywords internal
"_PACKAGE"

#' @importFrom checkmate assert
#' @importFrom checkmate assert_choice
#' @importFrom checkmate assert_disjunct
#' @importFrom checkmate assert_true
#' @importFrom checkmate check_class
#' @importFrom checkmate qassert
#' @importFrom checkmate test_class
#'
#' @importFrom data.table :=
#' @importFrom data.table %between%
#' @importFrom data.table %like%
#' @importFrom data.table copy
#' @importFrom data.table fcase
#' @importFrom data.table fifelse
#' @importFrom data.table setDT
#' @importFrom data.table tstrsplit
#'
#' @importFrom settings options_manager
#' @importFrom settings reset
#' @importFrom settings stop_if_reserved
#'
#' @importFrom units deparse_unit
#' @importFrom units as_units
#'
#' @importFrom wrapr %?%
#' @importFrom wrapr stop_if_dot_args
#'
NULL

#' Format isotopes in nuclear notation
#'
#' Convert chemical element or isotope from from hyphenated notation to
#' nuclear notation.
#'
#' We start with a character string in hyphenated form _X-A_ where _X_ is the
#' chemical symbol of an element and _A_ is its mass number.
#' For example, carbon 12 would be written as the string `"C-12"`.
#'
#' Given a character scalar, vector, or data frame column of isotopes in
#' this form, `format_nucl()` constructs the form `"$\\mathxx{^{A}X}$"` where
#' `\\mathxx` determines the font face: plain type is set by `\\mathrm`; italic
#' by `\\mathit`; bold by `\\mathbf`; sans serif by `\\mathsf`; and monospace
#' (typewriter text) by `\\mathtt`. The result includes markup delimiters `$...$`
#' for rendering (in an R markdown or Quarto markdown document) as an inline
#' code chunk.
#'
#' Unlike other functions in formatdown, `format_nucl()` does not support a
#' `size` argument.
#'
#' @param x Character. Hyphenated form of chemical elements or isotopes.
#'          Can be a single character string or a vector of strings. Must
#'          include a single hyphen between the element symbol and the mass
#'          number. For example, carbon 12 in hyphenated notation is the
#'          character string `"C-12"`.
#'
#' @param face `r param_face`
#'
#' @param ... `r param_dots`
#'
#' @param Z T/F add the atomic number. For details,
#'         see the help page for `formatdown_options()`.
#'
#' @param warn T/F issue warning for incorrect input `x`. For details,
#'         see the help page for `formatdown_options()`.
#'
#' @param delim The math-delimiter. For details, see the help page for
#'        `formatdown_options()`.
#'
#' @return A character vector of isotopes in nuclear notation with elements
#'         delimited as inline math markup in plain, italic, sans serif,
#'         bold, or monospace font face.
#'
#' @family format_*
#' @example man/examples/examples_format_nucl.R
#' @export
format_nucl <- function(x,
                        face = "plain",
                        ...,
                        Z = formatdown_options("Z"),
                        warn = formatdown_options("warn"),
                        delim = formatdown_options("delim")) {

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_nucl()")

  # Indicate these are not unbound symbols (R CMD check Note)
  hyphenated <- NULL
  atomic_number <- NULL
  mass_number <- NULL
  symbol <- NULL
  value <- NULL
  element_set <- element_set # built-in data set

  # Argument checks ---------------------------------------------------------

  # x: character, missing values prohibited, length at least 1
  checkmate::qassert(x, "S+")

  # Z & warn: logical, missing values prohibited, length exactly 1
  checkmate::qassert(Z, "B1")
  checkmate::qassert(warn, "B1")

  # face: character, not missing, length 1, element of set
  checkmate::qassert(face, "S1")
  checkmate::assert_choice(face, choices = c("plain",
                                             "italic",
                                             "bold",
                                             "sans",
                                             "mono"))

  # do the work ---------------------------------------------------------
  df <- data.frame(x)
  DT <- data.frame(tstrsplit(df$x,
                             split = '-',
                             fixed = TRUE,
                             names = c("symbol", "mass_number")))
  setDT(DT)

  # left-join atomic numbers
  DT <- element_set[DT, , on = c("symbol", "mass_number")]

  # identify rows with incorrect symbol or mass
  z_na <- is.na(DT$atomic_number)

  # replace NA atomic numbers with "Z" for output
  DT <- DT[z_na, atomic_number := "Z"]

  # error warning if any
  if (isTRUE(warn) & sum(z_na) > 0){

    # reassemble hyphenated form for warning statement
    err_rows <- DT[z_na, list(symbol, mass_number)]
    err_rows <- err_rows[, hyphenated := paste0(symbol, "-", mass_number)]
    err_vals <- err_rows[, list(hyphenated)]

    warning(paste("Incorrect chemical symbol or mass number in ", err_vals))
  }

  if (Z) {# include atomic number

    DT[, value := paste0("^{", mass_number  , "}",
                         "_{", atomic_number, "}",
                         symbol)]

  } else {# omit atomic number

    DT[, value := paste0("^{", mass_number, "}", symbol)]

  }

  # then call format_text()
  DT <- DT[, value := format_text(value, face = face)]

  # Convert to vector output
  DT <- DT[, (value)]
  DT[]

}

test_format_nucl <- function() {

  # library("data.table")
  # library("formatdown")

  # usage
  #
  # format_nucl(x,
  #             face = "plain",
  #             ...,
  #             Z = formatdown_options("Z"),
  #             warn = formatdown_options("warn"),
  #             delim = formatdown_options("delim"))



  # scalar correct behaviors
  x <- "C-12"

  ans101 <- "$\\mathrm{^{12}C}$"
  expect_equal(ans101, format_nucl(x))

  ans102 <- "$\\mathrm{^{12}_{6}C}$"
  expect_equal(ans102, format_nucl(x, Z = TRUE))

  ans103 <- "$\\mathit{^{12}C}$"
  expect_equal(ans103, format_nucl(x, face = "italic"))


  # vector correct behavior
  x <- c("C-12", "U-235")
  ans104 <-  c("$\\mathrm{^{12}C}$", "$\\mathrm{^{235}U}$")
  expect_equal(ans104, format_nucl(x))

  ans105 <- c("$\\mathrm{^{12}_{6}C}$",
              "$\\mathrm{^{235}_{92}U}$")
  expect_equal(ans105, format_nucl(x, Z = TRUE))


  # output class characteristics
  x <- c("C-12", "U-235")
  ans105a <- "character"
  expect_equal(ans105a, class(format_nucl(x)))

  ans105b <- 2
  expect_equal(ans105b, length(format_nucl(x)))


  # warning
  x <- "C-40"
  expect_warning(format_nucl(x))
  expect_warning(format_nucl(x, Z = TRUE))
  expect_warning(format_nucl(x, face = "italic"))

  x <- "carbon-12"
  expect_warning(format_nucl(x))
  expect_warning(format_nucl(x, Z = TRUE))
  expect_warning(format_nucl(x, face = "italic"))


  # omit warning, use symbols
  x <- "E-A"
  ans106 <- "$\\mathrm{^{A}E}$"
  expect_equal(ans106, format_nucl(x, warn = FALSE))
  expect_equal(ans106, suppressWarnings(format_nucl(x, warn = TRUE)))

  ans107 <- "$\\mathrm{^{A}_{Z}E}$"
  expect_equal(ans107, format_nucl(x, Z = TRUE, warn = FALSE))
  expect_equal(ans107, suppressWarnings(format_nucl(x, Z = TRUE, warn = TRUE)))


  # error due to improper inputs
  expect_error(format_nucl("C12"))
  expect_error(format_nucl("C-12-6"))
  expect_error(format_nucl(C12))
  expect_error(format_nucl(TRUE))
  expect_error(format_nucl(NULL))
  expect_error(format_nucl(NA_character_))






  # function output not printed
  invisible(NULL)
}

test_format_nucl()

# Show all options
formatdown_options()

# Store existing settings, including any changes made by the user
old_settings <- formatdown_options()

# View one option
formatdown_options()$delim

# View multiple options
formatdown_options("size", "delim")

# Change options
formatdown_options(size = "small", delim = "\\(")
formatdown_options("size", "delim")

# Reset to default values
formatdown_options(reset = TRUE)
formatdown_options("size", "delim")

# Reset options to those before this example was run
do.call(formatdown_options, old_settings)

# Option effects

# delim
x <- 101300
format_dcml(x)
# equivalent to
format_dcml(x, delim = c("$", "$"))
# built-in alternate
format_dcml(x, delim = "\\(")
# equivalent to
format_dcml(x, delim = c("\\(", "\\)"))

# size
format_dcml(x, size = "small")
# equivalent to
format_dcml(x, size = "\\small")
# other possible values
format_dcml(x, size = "scriptsize")
format_dcml(x, size = "large")
format_dcml(x, size = "huge")
# default NULL
format_dcml(x, size = NULL)
# renders equivalent to
format_dcml(x, size = "normalsize")

# decimal_mark
y <- 6.02214076E+10
format_sci(y, 5, decimal_mark = ".")
format_sci(y, 5, decimal_mark = ",")

# big_mark
format_dcml(y, 9)
format_dcml(y, 9, big_mark = "thin")
# equivalent to
format_dcml(y, 9, big_mark = "\\\\,")

# big_interval
format_dcml(y, 9, big_mark = "thin", big_interval = 3)
format_dcml(y, 9, big_mark = "thin", big_interval = 5)

# small_mark
z <- 1.602176634e-8
format_sci(z, 10)
format_sci(z, 10, small_mark = "thin")
format_sci(z, 10, small_mark = "\\\\,")
format_engr(z, 10, small_mark = "thin")

# small_interval
format_sci(z, 10, small_mark = "thin", small_interval = 3)
format_sci(z, 10, small_mark = "thin", small_interval = 5)
format_engr(z, 10, small_mark = "thin", small_interval = 5)

# whitespace in text
p <- "Hello world!"
format_text(p)
# equivalent to
format_text(p, whitespace = "\\\\ ")
# alternates
format_text(p, whitespace = "\\\\:")
format_text(p, whitespace = "\\\\>")

# whitespace in physical units expression
x <- pi
units(x) <- "m/s"
format_dcml(x)
# equivalent to
format_dcml(x, whitespace = "\\\\ ")

# multiply_mark
x <- 101300
format_engr(x, decimal_mark = ".", multiply_mark = "\\times")
format_engr(x, decimal_mark = ",", multiply_mark = "\\cdot")

#' Chemical elements
#'
#' Table of 118 chemical elements with name, symbol, atomic number, and
#' mass numbers. Elements with more than one possible mass number have
#' separate rows for each isotope.
#'
#' In "AZE" or "AZX" notation, "A" is the mass number, "Z" is the atomic number,
#' and "E" or "X" is the element symbol.
#'
#' @usage data(element_set, package = "formatdown")
#'
#' @format Classes data.table and data.frame with	`r nrow(element_set)`
#' observations of `r ncol(element_set)` variables:
#'
#'   \describe{
#'    \item{element}{Character, element name, hydrogen (atomic number 1)
#'                   through oganesson (atomic number 118).}
#'    \item{symbol}{Character, element symbol.}
#'    \item{atomic_number}{Character, 1-118.}
#'    \item{mass_number}{Character, 1-294.}
#'    }
"element_set"

#' Air density measurements
#'
#' Table of air properties at room temperature and pressure, simulating multiple
#' measurements at approximately steady state,
#'
#' @usage data(air_meas, package = "formatdown")
#'
#' @format Classes data.table and data.frame with	`r nrow(air_meas)` observations
#' of `r ncol(air_meas)` variables:
#'
#'   \describe{
#'    \item{date}{"Date" class format "YYYY-MM-DD".}
#'    \item{trial}{Character, label "a" through "e".}
#'    \item{humid}{Factor, humidity, "low", "med", or "high."}
#'    \item{temp}{Numeric, measured temperature (K).}
#'    \item{pres}{Numeric, measured atmospheric pressure (Pa).}
#'    \item{sp_gas}{Numeric, specific gas constant in mass form \eqn{R_{sp}},
#'          ideal gas reference value, (J kg\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}}).}
#'    \item{dens}{Numeric, calculated air density
#'          \eqn{\rho} = \eqn{p}\eqn{R_{sp}^{-1}}{^{-1}}\eqn{T^{-1}}{^{-1}}
#'          (kg m\eqn{^{-3}}{^{-3}}).} }
"air_meas"

#' Properties of standard atmosphere
#'
#' Table of atmospheric properties as a function of altitude, sea level to
#' 1000 km.
#'
#' @usage data(atmos, package = "formatdown")
#'
#' @format Classes data.table and data.frame with `r nrow(atmos)` observations
#' of `r ncol(atmos)` variables:
#'
#' \describe{
#'  \item{alt}{Numeric, altitude (km)}
#'  \item{temp}{Numeric, air temperature (K)}
#'  \item{pres}{Numeric, atmospheric pressure (Pa)}
#'  \item{dens}{Numeric, air density (kg m\eqn{^{-3}}{^{-13}})}
#'  \item{sound}{Numeric, speed of sound (m/s)}
#' }
#'
#' @source \emph{Marks' Standard Handbook for Mechanical Engineers 9/e} (1987)
#'         E.A. Avallone and T. Baumeister (ed.),
#'         "Table 4.2.2 International Standard Atmosphere",
#'         pp. 4-38,  McGraw-Hill, NY.
#'
"atmos"

#' Properties of metals
#'
#' Table of mechanical and thermal properties of selected metals.
#'
#' @usage data(metals, package = "formatdown")
#'
#' @format Classes data.table and data.frame with	`r nrow(metals)` observations
#' of `r ncol(metals)` variables:
#'
#' \describe{
#'  \item{metal}{Character, name of material}
#'  \item{dens}{Numeric, density (kg m\eqn{^{-3}}{^{-3}})}
#'  \item{thrm_exp}{Numeric, coefficient of thermal expansion
#'       (m m\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}})}
#'  \item{thrm_cond}{Numeric, thermal conductivity (W m\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}})}
#'  \item{elast_mod}{Numeric, modulus of elasticity (Pa)}
#' }
#'
#' @source \emph{Marks' Standard Handbook for Mechanical Engineers 9/e} (1987)
#'         E.A. Avallone and T. Baumeister (ed.),
#'         "Basic Properties of Several Metals",
#'         pp. 6-11,  McGraw-Hill, NY.
#'
"metals"

#' Properties of water
#'
#' Table of water properties at atmospheric pressure as a function of
#' temperature.
#'
#' @usage data(water, package = "formatdown")
#'
#' @format Classes data.table and data.frame with `r nrow(water)` observations
#' of `r ncol(water)` variables:
#'
#' \describe{
#'  \item{temp}{Numeric, temperature (K)}
#'  \item{dens}{Numeric, density (kg m\eqn{^{-3}}{^{-3}})}
#'  \item{sp_wt}{Numeric, specific weight (N m\eqn{^{-3}}{^{-3}})}
#'  \item{visc}{Numeric, dynamic viscosity (Pa s)}
#'  \item{bulk_mod}{Numeric, bulk modulus (Pa)}
#' }
#'
#' @source E. Maurer E and I. Embry (2022) \emph{hydraulics: Basic Pipe and
#' Open Channel Hydraulics}, R package ver. 0.6.0,
#' \url{https://edm44.github.io/hydraulics/}.
#'
"water"

# Internal function not exported:

formatdown_opts <- settings::options_manager(
  delim          = "$",
  size           = NULL,
  decimal_mark   = ".",
  big_mark       = "",
  big_interval   = 3,
  small_mark     = "",
  small_interval = 5,
  whitespace     = "\\\\ ",
  multiply_mark  = "\\times",
  Z              = FALSE,
  warn           = TRUE
)

# User function is exported:

#' Get and set function arguments via options
#'
#' Changes the default values of function arguments which affect the markup and
#' appearance of formatdown results.
#'
#' Global options are provided for arguments that users would likely prefer to
#' set once in a document instead of repeating in every function call. For
#' example, some users prefer a comma decimal marker (",") throughout a
#' document.
#'
#' Globally-set arguments can be overridden locally by assigning them in a
#' function call.
#'
#' The arguments that can be set with this function are as follows:
#'
#' - `delim`:          `r param_delim`
#' - `size`:           `r param_size`
#' - `decimal_mark`:   `r param_decimal_mark`
#' - `big_mark`:       `r param_big_mark`
#' - `big_interval`:   `r param_big_interval`
#' - `small_mark`:     `r param_small_mark`
#' - `small_interval`: `r param_small_interval`
#' - `whitespace`:     `r param_whitespace`
#' - `multiply_mark`:  `r param_multiply_mark`
#' - `Z`:              `r param_Z`
#' - `warn`:           `r param_warn`
#'
#'
#' @param reset Logical vector of length 1; if TRUE, reset all options to their
#'     default values.
#'
#' @param ... One or more `name = value` pairs to set values; or one or more
#'     quoted option names to get values.
#'
#' @return Nothing; used for its side-effect.
#' @example man/examples/examples_formatdown_options.R
#' @export
formatdown_options <- function(..., reset = FALSE) {

  # reset is Boolean, not missing, length 1
  checkmate::qassert(reset, "B1")

  if (reset) {
    settings::reset(formatdown_opts)
  } else {
    formatdown_opts(...)
  }
}

# default arguments
x <- "C-12"
format_nucl(x)

# add atomic number
format_nucl(x, Z = TRUE)

# vector of isotopes
x <- c("C-12", "Fe-54", "U-238")
format_nucl(x, Z = TRUE)

# turn off the warning when using symbols
x <- "X-A"
format_nucl(x, Z = TRUE, warn = FALSE)

# change the typeface
x <- "C-12"
format_nucl(x, face = "bold")

# obtain all possible mass numbers for an element
element_set[symbol == "Ca"]

# input: single number
x <- 6.0221E+23
format_sci(x)

# input: units class
x <- 103400
units(x) <- "N m2 C-2"
format_sci(x)

# input: vector
data("metals", package = "formatdown")
x <- metals$dens
format_sci(x)

# significant digits
x <- 9.75358e+5
format_sci(x, 2)
format_sci(x, 3)
format_sci(x, 4)

# input: data frame
x <- metals[, c("thrm_exp", "thrm_cond")]
as.data.frame(apply(x, 2, format_sci, digits = 3))

# format_sci() same as format_numbers(..., format = "sci")
x <- 6.0221E+23
format_sci(x)
format_numbers(x, format = "sci")

# omit_power
x <- 103400
format_sci(x, omit_power = c(-1, 2)) # default
format_sci(x, omit_power = c(-1, 5))
format_sci(x, omit_power = 5) # equivalent to omit_power = c(5, 5)

# omit_power = NULL, power-of-ten notation for all elements
x <- c(1.2, 103400)
format_sci(x)
format_sci(x, omit_power = NULL)

# set_power overrides default scientific exponent
x <- 103400
format_sci(x)
format_sci(x, set_power = 4)

# set_power overrides omit_power
x <- 103400
format_sci(x)
format_sci(x, omit_power = 5)
format_sci(x, omit_power = 5, set_power = 4)

# R code used for documentation

param_x <- 'Number or numbers to be formatted. Can be a single number, a vector,
        or a column of a data frame.'

param_digits <- 'Integer from 1 through 20 that controls the number of
        significant digits in printed numeric values. Passed to `signif()`.
        Default is 4.'

param_format <- 'Character, length 1, defines the type of notation. Possible
        values are `"engr"` (default) for engineering power-of-ten notation,
        `"sci"` for scientific power-of-ten notation, and `"dcml"` for decimal
        notation.'

param_dots <- 'Not used for values; forces subsequent arguments to be
        referable only by name.'

param_omit_power <- 'Numeric vector `c(p, q)` with `p <= q`, specifying
        the range of exponents over which power-of-ten notation is omitted in
        either scientific or engineering format. Default is
        `c(-1, 2)`. If a single value is assigned, i.e., `omit_power = p`, the
        argument is interpreted as `c(p, p)`. If `NULL` or `NA`, all elements
        are formatted in power-of-ten notation. Argument is overridden by
        specifying `set_power` or decimal notation.'

param_set_power <- 'Integer, length 1. Formats all values in `x` with the same
        power-of-ten exponent. Default NULL. Overrides `format` and `omit_power`
        arguments.'

param_delim <- 'Character, length 1 or 2, to define the left and right math
        markup delimiters. The default setting, `delim = "$"`, produces
        left and right delimiters `$...$`. The alternate built-in
        setting, `delim = "\\\\("`, produces left and right delimiters
        `\\\\( ... \\\\)`. Custom delimiters can be assigned in a vector of
        length 2 with left and right delimiter symbols, e.g.,
        `c("\\\\[", "\\\\]")`. Special characters typically must be escaped.'

param_size <- 'Character, length 1, to assign a font size. If not empty, adds
        a font size macro to the markup inside the math delimiters. Possible
        values are `"scriptsize"`, `"small"`, `"normalsize"`, `"large"`, and
        `"huge"`. One may also assign the equivalent LaTeX-style markup itself,
        e.g., `"\\\\scriptsize"`, `"\\\\small"`, etc. Default is NULL.'

param_decimal_mark <- 'Character, length 1, to assign the decimal marker.
        Possible values are a period `"."` (default) or a comma `","`. Passed
        to `formatC(decimal.mark)`.'

param_big_mark <- 'Character, length 1, used as the mark between every
        `big_interval` number of digits to the left of the decimal marker to
        improve readability. Possible values are empty `""` (default) or
        `"thin"` to produce a LaTeX-style thin, horizontal space. One may also
        assign the thin-space markup itself `"\\\\\\\\,"`.
        Passed to `formatC(big.mark)`.'

param_big_interval <- 'Integer, length 1, that defines the number of digits
        (default 3) in groups separated by `big_mark`. Passed to
        `formatC(big.interval)`.'

param_small_mark <- 'Character, length 1, used as the mark between every
        `small_interval` number of digits to the right of the decimal marker to
        improve readability. Possible values are empty `""` (default) or
        `"thin"` to produce a LaTeX-style thin, horizontal space. One may also
        assign the thin-space markup itself `"\\\\\\\\,"`.
        Passed to `formatC(small.mark)`.'

param_small_interval <- 'Integer, length 1, that defines the number of digits
        (default 5) in groups separated by `small_mark`. Passed to
        `formatC(small.interval)`.'

param_whitespace <- 'Character, length 1, to define the LaTeX-style
        math-mode macro to preserve a horizontal space between words of text or
        between physical-unit abbreviations when formatting numbers of class
        "units". Default is `"\\\\\\\\ "`. Alternatives include `"\\\\\\\\:"` or
        "`\\\\\\\\>`".'

param_multiply_mark <- 'Character, length 1, to define the multiplication symbol
        in power of ten notation. Possible values are `"\\\\times"` (default)
        or a half-high dot `"\\\\cdot"` which is often used when the decimal
        mark is a comma.'

param_Z <- 'Logical, length 1, default FALSE, used to determine whether or
        not the atomic number subscript is used in formatting nuclear
        notation of isotopes.'

param_warn <- 'Logical, length 1, default TRUE, used to determine whether or
        not a warning is issued when an input for a chemical element in
        hyphenated notation contains an error in the element symbol or the
        mass number of the isotope.'

param_face <- 'Font face. Determines the font face macro inside the math
        delimiters. Possible values are `"plain"` (default), `"italic"`,
        `"bold"`, `"sans"`, or `"mono"`. One may assign instead the
        corresponding LaTeX-style markup itself, e.g., `\\mathrm`,
        `\\mathit`, `\\mathbf`, `\\mathsf`, or `\\mathtt`.'

# Internal functions, not exported

# Typeface
get_math_markup <- function(face) {

  # Default is plain
  math_markup  <- "\\mathrm"

  # Allow shorthand and markup style
  if (isTRUE(!is.na(face)) & isTRUE(!is.null(face))) {
    if(face == "plain"  | face == "\\mathrm") math_markup  <- "\\mathrm"
    if(face == "italic" | face == "\\mathit") math_markup  <- "\\mathit"
    if(face == "bold"   | face == "\\mathbf") math_markup  <- "\\mathbf"
    if(face == "sans"   | face == "\\mathsf") math_markup  <- "\\mathsf"
    if(face == "mono"   | face == "\\mathtt") math_markup  <- "\\mathtt"
  }
  return(math_markup)
}

#' Format text
#'
#' Convert a character vector to "math text" delimited for rendering as inline
#' equations in an R markdown document. Particularly useful for matching the
#' font face of character columns to that of numerical columns in a table.
#'
#' Given a scalar, vector, or data frame column, `format_text()` converts its
#' argument to a character string of the form `"$\\mathxx{a}$"` where `a`
#' is the element to be formatted and `\\mathxx` determines the font face:
#' plain type is set by `\\mathrm`; italic by `\\mathit`;
#' bold by `\\mathbf`; sans serif by `\\mathsf`; and monospace (typewriter
#' text) by `\\mathtt`. All strings include markup delimiters `$...$` for
#' rendering (in an R markdown or Quarto markdown document) as an inline
#' equation.
#'
#' @param x Text to be formatted. Can be a single string,
#'          a vector, or a column of a data frame.
#'
#' @param ... `r param_dots`
#'
#' @param face `r param_face`
#'
#' @param size,delim,whitespace Used to format the math-delimited character
#'   strings. For details, see the help page for `formatdown_options()`.
#'
#' @return A character vector with elements delimited as inline math markup
#'  in plain, italic, sans serif, bold, or monospace font face.
#'
#' @family format_*
#' @example man/examples/examples_format_text.R
#' @export
format_text <- function(x,
                        face = "plain",
                        ...,
                        size = formatdown_options("size"),
                        delim = formatdown_options("delim"),
                        whitespace = formatdown_options("whitespace")) {

  # Overhead ----------------------------------------------------------------

  # Arguments after dots must be named
  wrapr::stop_if_dot_args(substitute(list(...)), "format_text()")

  # Indicate these are not unbound symbols (R CMD check Note)
  value <- NULL
  math_markup <- NULL
  size_markup <- NULL

  # Assign arguments -------------------------------------------------------

  # size
  if (is_null_or_na(size)) size <- NA_character_

  # Argument checks ---------------------------------------------------------

  # x: coercible to character
  checkmate::qassert(as.character(x), "s*")

  # face: character, not missing, length 1, element of set
  checkmate::qassert(face, "S1")
  checkmate::assert_choice(
    face,
    choices = c("plain", "italic", "bold", "sans", "mono", "\\mathrm", "\\mathit", "\\mathbf", "\\mathsf", "\\mathtt")
    )

  # Assertions on options (in utils.R)
  param_assert_delim(delim)
  param_assert_size(size)
  param_assert_whitespace(whitespace)

  # Initial processing ------------------------------------------------------

  # Assign markup parameters
  size_markup <- get_size_markup(size)
  math_markup <- get_math_markup(face)

  # Begin ------------------------------------------------------

  # Convert vector to data.table for processing
  DT <- data.table::copy(data.frame(x))
  setDT(DT)

  # To retain spaces within x, sub LaTeX "\:"
  DT[, value := as.character(x)]
  DT[, value := gsub(" ", whitespace, value)]
  DT[is.na(x), value := NA_character_]

  # Add math markup
  DT[, value := paste0(math_markup, "{", value, "}")]

  # Prep for output
  output_vector <- prepare_output_vector(DT, size_markup, delim)
  output_vector[]
  return(output_vector)
}

test_format_text<- function() {

  # All tests, e.g., expect_equal, expect_error, etc., are from the
  # tinytest package. Possible test functions listed at
  # https://github.com/markvanderloo/tinytest/blob/master/pkg/README.md

  # usage
  # format_text(x, ..., face = NULL, size = NULL, delim = "$")
  # arguments after dots must be named

  # correct behaviors

  # character vector
  x <- c("a", "b", "c")
  ans1 <- c("$\\mathrm{a}$", "$\\mathrm{b}$", "$\\mathrm{c}$")
  expect_equal(format_text(x), ans1)

  # character vector
  x <- c("1.2E-3", "3.4E+0", "5.6E+3")
  ans2 <- c("$\\mathrm{1.2E-3}$", "$\\mathrm{3.4E+0}$", "$\\mathrm{5.6E+3}$")
  expect_equal(format_text(x), ans2)

  # numeric coerced to text
  x <- c(1.2, 2.3, 3.4)
  ans3 <- c("$\\mathrm{1.2}$", "$\\mathrm{2.3}$", "$\\mathrm{3.4}$")
  expect_equal(format_text(x), ans3)

  # logical coerced to text
  x <- c(TRUE, FALSE, TRUE)
  ans4 <- c("$\\mathrm{TRUE}$", "$\\mathrm{FALSE}$", "$\\mathrm{TRUE}$")
  expect_equal(format_text(x), ans4)

  # NA treated as character, NULL ignored
  x <- c(NA, NULL)
  ans5a <- c("$\\mathrm{NA}$")
  expect_equal(format_text(x), ans5a)

  # NULL alone, character, length 0
  x <- NULL
  ans5b <- 0
  expect_equal(length(format_text(x)), ans5b)

  # face options c("plain", "italic", "bold", "sans", "mono")
  x <- "A"
  expect_equal(format_text(x), format_text(x, face = "plain"))
  ans6a <-"$\\mathrm{A}$"
  expect_equal(format_text(x, face = "plain"),  ans6a)
  ans6b <-"$\\mathit{A}$"
  expect_equal(format_text(x, face = "italic"), ans6b)
  ans6c <-"$\\mathbf{A}$"
  expect_equal(format_text(x, face = "bold"),   ans6c)
  ans6d <-"$\\mathsf{A}$"
  expect_equal(format_text(x, face = "sans"),   ans6d)
  ans6e <-"$\\mathtt{A}$"
  expect_equal(format_text(x, face = "mono"),   ans6e)

  # Data frame, one column as vector
  x   <- air_meas[, (humid)]
  ans10 <- c("$\\mathrm{low}$",
           "$\\mathrm{high}$",
           "$\\mathrm{med}$",
           "$\\mathrm{low}$",
           "$\\mathrm{high}$")
  expect_equal(format_text(x), ans10)

  # ERRORS  ------------------------
  x <- "hello world"

  # arguments of incorrect class
  expect_error(format_text(x = air_meas))
  expect_error(format_text(x, face = TRUE))

  # arguments not among allowed choices
  expect_error(format_text(x, face = c("plain", "bold")))
  expect_error(format_text(x, face = "bold.italic"))

  # arguments after dots must be named
  expect_error(format_text(x, "plain", "small"))
  expect_error(format_text(x, "plain", size = "small", "$"))
  expect_error(format_text(x, "plain", size = "small", delim = "$", "\\\\>"))

  # function output not printed
  invisible(NULL)
}

test_format_text()

---
title: "Format numbers"
vignette: >
  %\VignetteIndexEntry{Format numbers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output: rmarkdown::html_vignette
bibliography: ../inst/REFERENCES.bib
csl: ../inst/information-science-and-technology.csl
link-citations: yes
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  tidy = TRUE
)

# to knit "child" Rmd files
knitr::opts_knit$set(root.dir = "../")

library(formatdown)
library(data.table)
library(knitr)

options(
  datatable.print.nrows = 15,
  datatable.print.topn = 3,
  datatable.print.class = TRUE
)
```


```{r}
#| echo: false

formatdown_options(size = "small")

prefix_1 <- c("peta", "tera", "giga", "mega", "kilo")

symbol_1 <- c("P", "T", "G", "M", "k")
symbol_1 <- format_text(symbol_1, face = "italic")

x <- 10^seq(from = 15, to = 3, by = -3)
value_1 <- format_numbers(x, digits = 1, format = "engr", omit_power = NULL)
value_1 <- sub("1 \\\\times ", "", value_1)

prefix_2 <- c("milli", "micro", "nano", "pico", "femto")

symbol_2 <- c("m", "\\mu", "n", "p", "f")
symbol_2 <- format_text(symbol_2, face = "italic")

x <- 10^seq(from = -3, to = -15, by = -3)
value_2 <- format_numbers(x, 1, omit_power = NULL)
value_2 <- sub("1 \\\\times ", "", value_2)

DT <- data.table(prefix_1, symbol_1, value_1, prefix_2, symbol_2, value_2)
knitr::kable(DT,
  align = "lcllcl",
  col.names = rep(c("Prefix", "Symbol", "Value"), 2)
)
```

The rationale for the formatdown package is formatting numbers in power-of-ten notation in inline R code or tabulated columns of data frames. Other features of the package provide tools for typesetting non-power-of-ten columns to match. In this vignette, we discuss the primary formatting function `format_numbers()` and its convenience wrappers for scientific, engineering, and decimal notation.

## Types of notation

Notation to represent large and small numbers depends on the mode of communication. In a computer script, for example, we might encode the Avogadro constant as `N_A = 6.0221*10^23`. The asterisk (\*) and caret (\^) in this expression, however, communicate *instructions to a computer*, not *syntactical mathematics*. And while scientific E-notation (`6.0221E+23`) has currency in some discourse communities, power-of-ten notation, e.g., $\small N_A = 6.0221 \times 10^{23}$, is the conventional format for professional technical communication. 

Power-of-ten notation is expressed,   

$$
\small a \times 10^n, 
$$

where $\small a$ is the coefficient in decimal form and the exponent $\small n$ is an integer. Two formats are in common use [@Chase:2021, 63--67]: 

- *scientific:* &nbsp; $\small 1\leq{|a|} < 10$, e.g., $\small N_A =$ `r format_sci(6.0221E+23, 5)`. 

- *engineering:* &nbsp; $\small  1\leq{|a|} < 1000$ and $\small n$ is a multiple of 3, e.g., $\small N_A =$ `r format_engr(6.0221E+23, 5)`. 

The utility of the engineering form follows from the SI prefixes for physical units such as "mega-", "kilo-", "milli-", etc., corresponding to powers of 10 that are integer multiples of three. 

<br>

*Notes on syntax.* &nbsp; Programming symbols are not necessarily mathematical symbols:

- asterisk (\*). Programming symbol for multiplication, e.g., `x = a * b` or `y = a * (b + c)`. In the grammar of mathematics, multiplication is indicated by the symbol ($\times$) when needed.   

- caret (\^). Programming symbol for exponentiation, e.g., `x = y^2` or `z = 10^-3`. In the grammar of mathematics, exponents are typeset as superscripts, e.g., $\small x = y^2$ or $\small z = 10^{-3}$.

- multiplication ($\small\times$). Mathematical symbol for multiplication, not to be confused with the letter "x". Generally omitted when the meaning is clear, e.g., $\small x=ab$ or $\small y=a(b+c)$, but conventionally included in power-of-ten notation, e.g., $\small 6.0221 \times 10^{23}$. When a comma is used as the decimal marker, multiplication may be indicated by the half-high dot, e.g., $\small 6,0221 \cdot 10^{23}$.

<br>

*Decimal subsets.* &nbsp; In a vector of numbers formatted in power-of-ten form, the decimal form may be preferred for any subset of values with exponents near zero, e.g., $\small n \in \{-1, 0, 1, 2\}$. 


```{r}
#| echo: false

x <- 3.12 * 10^seq(-3, 4)

sci <- format_sci(x, 3, omit_power = NA)
sci_omit <- format_sci(x, 3)
DT <- data.table(sci, sci_omit)

knitr::kable(DT,
  align = "r",
  caption = "Decimal form may be preferred for a subset",
  col.names = c(
    "scientific notation",
    "subset in decimal form"
  )
)
```

<br>

*Decimal columns.* &nbsp; A table of numeric information can include columns formatted in both power-of-ten notation and decimal notation. For example, a  table of atmospheric properties shown below has altitude in integer form, temperature in decimal form,  and density in power-of-ten engineering notation (except for those values with exponents near zero). 

```{r}
#| echo: false

DT <- atmos[alt < 150, .(alt, temp, dens)]

DT$alt <- format_dcml(DT$alt, 2)
DT$temp <- format_dcml(DT$temp, 5)
DT$dens <- format_engr(DT$dens, 3)
knitr::kable(DT,
  align = "r",
  caption = "Properties of the atmosphere",
  col.names = c(
    "Altitude (km)",
    "Temperature (K)",
    "Density (kg/m$^3$)"
  )
)

formatdown_options(reset = TRUE)
```

The purpose of the decimal format in formatdown is to match the font face and size of decimal columns to those of the power-of-ten columns. If no power-of-ten columns are used, of course, decimal columns can be displayed as-is or formatted using other R tools. 

<br>

*Packages.* &nbsp; If you are writing your own script to follow along, we use the following packages in this vignette. Data frame operations are performed with data.table syntax. Some users may wish to translate the examples to use  base R or dplyr syntax.  

```{r}
#| echo: true
#| eval: false

library("formatdown")
library("data.table")
library("knitr")
```

## Markup {#markup}

We format numbers as inline math expressions  delimited by `$ ... $` or the optional `\( ... \)`. For example, the Avogadro constant is marked up as

        $6.0221 \times 10^{23}$, 

where the `\times` macro creates the multiplication symbol ($\small\times$). This math markup, as an inline equation in an R markdown document, renders as: $\small 6.0221 \times 10^{23}$. To *program* the markup, however, we enclose it in quote marks as a character string, that is, 

        "$6.0221 \\times 10^{23}$", 

which requires us to "escape" the backslash in `\times` by adding an extra backslash. When the optional font size argument is assigned, formatdown adds a LaTeX-style sizing macro such as `\small` or `\large`, for example, 

        "$\\small 6.0221 \\times 10^{23}$", 

where again the markup includes an extra backslash.  



## `format_sci()` {#format_sci}

Converts numbers to character strings in power-of-ten form, 

        "$a \\times 10^{n}$" 

where $\small a$ is the coefficient and $\small n$ is the exponent. `format_sci()` is a wrapper for the more general function `format_numbers()`. For a subset of values with exponents near zero, e.g., $\small n \in \{-1, 0, 1, 2\}$, the output is in decimal form, 

        "$a$"

<br>

*Usage.* &nbsp; 

```r
format_sci(x,
           digits = 4,
           ...,
           omit_power = c(-1, 2),
           set_power = NULL,
           delim          = formatdown_options("delim"),
           size           = formatdown_options("size"),
           decimal_mark   = formatdown_options("decimal_mark"),
           small_mark     = formatdown_options("small_mark"),
           small_interval = formatdown_options("small_interval"), 
           whitespace     = formatdown_options("whitespace"))
```

- Arguments before the dots do not have to be named if argument order is maintained. 
- Arguments after the dots (`...`) must be named. 
- The arguments assigned via `formatdown_options()` can be reset by the user  locally in a function call or globally using `formatdown_options()`. 
 

<br>

*Examples.*  &nbsp; These early examples are shown with default arguments. Arguments are explored more fully starting with [Numeric input](#numeric-input) section. 

```{r}
# 1. Avogadro constant
L <- 6.0221E+23
format_sci(L)

# 2. Elementary charge
e <- 1.602176634e-19
format_sci(e)
```

Examples 1 and 2 (in inline code chunks) render as,    

1. The Avogadro constant is $\small L =$ `r format_sci(L, size = "small")` $\small \mathit{mol}^{-1}$.
1. The elementary charge constant is $\small e =$ `r format_sci(e, size = "small")` $\small C$. 




## `format_engr()` {#format_engr}

Similar to `format_sci()` except using engineering notation, i.e., exponents are multiples of 3. 

<br>

*Usage.* 

```r
format_engr(x,
            digits = 4,
            ...,
            omit_power = c(-1, 2),
            set_power = NULL,
            delim          = formatdown_options("delim"),
            size           = formatdown_options("size"),
            decimal_mark   = formatdown_options("decimal_mark"),
            small_mark     = formatdown_options("small_mark"),
            small_interval = formatdown_options("small_interval"), 
            whitespace     = formatdown_options("whitespace"))
```

<br>

*Examples.*  &nbsp; (with default arguments)

```{r}
# 3. Avogadro constant
format_engr(L)

# 4. Elementary charge
format_engr(e)
```

Examples 3 and 4 render as,   

3. The Avogadro constant is $\small L =$ `r format_engr(L, size = "small")` $\small \mathit{mol}^{-1}$.
3. The elementary charge constant is $\small e =$ `r format_engr(e, size = "small")` $\small C$. 




## `format_dcml()` {#format_dcml}

A wrapper for the more general function `format_numbers()`; converts numbers to  character strings in decimal form, 

        "$a$"

 where $\small a$ is the decimal value.  

<br>

*Usage.* 

```r
format_dcml(x,
            digits = 4,
            ...,
            size           = formatdown_options("size"),
            delim          = formatdown_options("delim"),
            decimal_mark   = formatdown_options("decimal_mark"),
            big_mark       = formatdown_options("big_mark"),
            big_interval   = formatdown_options("big_interval"),
            small_mark     = formatdown_options("small_mark"),
            small_interval = formatdown_options("small_interval"), 
            whitespace     = formatdown_options("whitespace"))
```

<br>

*Examples.*  &nbsp; (with default arguments)

```{r}
# 5. Speed of light in a vacuum
c <- 299792458
format_dcml(c)

# 6. Molar gas constant
R <- 8.31446261815324
format_dcml(R)
```

Examples 5 and 6 render as,  

5. The speed of light in a vacuum is $\small c =$ `r format_dcml(c, size = "small")` $\small\mathit{m/s}$.
5. The molar gas constant is $\small R =$ `r format_dcml(R, size = "small")` $\small\mathit{J}\cdot\mathit{K}^{-1}\mathit{mol}^{-1}$. 





## `format_numbers()` {#format_numbers}

`format_numbers()` is the general-purpose formatting function called by `format_sci()`, `format_engr()`, and `format_dcml()`. The general function can be used instead of the convenience functions simply by setting its `format` argument to `"sci"`, `"engr"` (default), or `"dcml"`. 


<br>

*Usage.* 

```r
format_numbers(x,
               digits = 4,
               format = "engr",
               ...,
               omit_power = c(-1, 2),
               set_power = NULL,
               delim          = formatdown_options("delim"),
               size           = formatdown_options("size"),
               decimal_mark   = formatdown_options("decimal_mark"),
               big_mark       = formatdown_options("big_mark"),
               small_mark     = formatdown_options("small_mark"),
               big_interval   = formatdown_options("big_interval"),
               small_interval = formatdown_options("small_interval"), 
               whitespace     = formatdown_options("whitespace"))
```


*Examples.*  &nbsp; Reproducing some of the earlier examples using `format_numbers()`.

```{r}
# 7. Scientific
format_numbers(L, format = "sci")

# 8. Engineering
format_numbers(e, format = "engr")

# 9. Decimal
format_numbers(R, format = "dcml")
```

Examples 7--9 render as,  

7. The Avogadro constant is $\small L =$ `r format_numbers(L, format = "sci", size = "small")` $\small \mathit{mol}^{-1}$. 
7. The elementary charge constant is $\small e =$ `r format_numbers(e, format = "engr", size = "small")` $\small C$. 
7. The molar gas constant is  $\small R =$ `r format_dcml(R, size = "small")` $\small\mathit{J}\cdot\mathit{K}^{-1}\mathit{mol}^{-1}$. 





## Numeric input {#numeric-input}

This section begins our detailed discussion of arguments.  

*Scalar input.* &nbsp; Generally used with inline R code. For example, the following R markdown sentence, which includes some math markup and some inline R code,  

````{verbatim}
    The Avogadro constant is $L = $ `r format_sci(L)` $\mathit{mol}^{-1}$. 
````

renders as: The Avogadro constant is $\small L =$ `r format_sci(L, size = "small")` $\small\mathit{mol}^{-1}$.

<br>

*Vector.* &nbsp; A vector of numbers (or a data frame column) is marked up as follows, 

```{r}
# 10. Sample vector
x <- c(2.3333e-5, 3.4444e-4, 5.2222e-2, 6.3333e-1, 8.1111e+1, 9.2222e+2, 2.4444e+4, 3.1111e+5, 4.2222e+6)
format_engr(x)
```

In a table, the output renders as,   

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
DT <- data.table(x, format_engr(x))
knitr::kable(DT,
  align = "r",
  col.names = c("Unformatted", "Engr notation"),
  caption = "Example 10."
)
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```

For values with exponents $\small n\in\{-1, 0, 1, 2\}$, the default format is decimal; see [Excluding exponents](#excluding-exponents). 







## Units input

The units R package (website: [Measurement Units for R](https://r-quantities.github.io/units/)) provides measurement units for R vectors, converting vectors of class "numeric" to class "units" [@Pebesma+Mailund+Hiebert:2016:units]. For example

```{r}
# Number
x <- 10320
class(x)

# Convert to units class
units(x) <- "m"
x
class(x)

# Operations are reflected in the values and its units
y <- x^2
y

# Unit conversion is supported
z <- y
z
units(z) <- "ft^2"
z
```

If an input argument to `format_numbers()` (or its convenience functions) is of class "units", formatdown attempts to extract the units character string, format the number in the expected way, and append a units character string to the result. For example, 

```{r}
# 11. Units-class inputs
format_sci(x)
format_sci(y)
format_sci(z)
```

Example 11 renders as, 

- `r format_sci(x, size = "small")`
- `r format_sci(y, size = "small")`
- `r format_sci(z, size = "small")`

More complicated units can be managed. For example the Newtonian gravitational constant could be formatted as follows, where the exponents in the units definition are given in "implicit" form, that is, where $\small m^3 kg^{-1} s^{-2}$ is represented by `"m3 kg-1 s-2"`. 

```r
    G        <- 6.6743e-11
    units(G) <- "m3 kg-1 s-2"
    format_sci(G)
```


Applying a similar procedure to several physical constants and collecting the results in a data frame yields, 

```{r}
#| echo: false
c <- 299792458
units(c) <- "m/s"
format_c <- format_sci(c, size = "small")

h <- 6.62607015e-34
units(h) <- "J/Hz"
format_h <- format_sci(h, size = "small")

mu <- 1.25663706212e-6
units(mu) <- "N A-2"
format_mu <- format_sci(mu, size = "small")

G <- 6.67430e-11
units(G) <- "m3 kg-1 s-2"
format_G <- format_sci(G, size = "small")

ke <- 8.9875517923e+9
units(ke) <- "N m2 C-2"
format_ke <- format_sci(ke, size = "small")

sigma <- 5.67037442e-8
units(sigma) <- "W m-2 K-4"
format_sigma <- format_sci(sigma, size = "small")

symbol <- c(
  "$\\small c$",
  "$\\small h$",
  "$\\small \\mu_0$",
  "$\\small G$",
  "$\\small k_e$",
  "$\\small \\sigma$"
)

quantity <- format_text(
  c(
    "speed of light in a vacuum",
    "Planck constant",
    "vacuum magnetic permeability",
    "Newtonian gravitational constant",
    "Coulomb constant",
    "Stefan-Boltzmann constant"
  ),
  size = "small"
)

formatted_value <- c(
  format_c,
  format_h,
  format_mu,
  format_G,
  format_ke,
  format_sigma
)

DT <- data.table(symbol, quantity, formatted_value)
knitr::kable(DT)
```


This table is constructed simply to illustrate how formatdown returns a variety of units-class values with  units appended to the formatted number. 

In a typical application, however, the numbers in a column have the same physical units and are formatted as a vector. For example, 


```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# Example 12
DT <- air_meas[, .(temp, pres, sp_gas, dens)]

# Examine data
DT[]

# Assign units
units(DT$temp) <- "K"
units(DT$pres) <- "Pa"
units(DT$sp_gas) <- "J kg-1 K-1"
units(DT$dens) <- "kg m-3"

# Format one column at a time
DT$temp <- format_dcml(DT$temp)
DT$pres <- format_engr(DT$pres)

# Or format multiple columns in one pass
cols <- c("sp_gas", "dens")
DT[, (cols) := lapply(.SD, format_dcml), .SDcols = cols]

knitr::kable(DT, align = "r", caption = "Example 12.")
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```





## Significant digits {#significant-digits}

Significant digits are applied to the input argument using the base R function `signif()` before additional formatting is applied.  For example, 


```{r}
# 13. Significant digits
format_sci(e, digits = 5)
format_sci(e, digits = 4)
format_sci(e, digits = 3)
```

Example 13 renders as, 

- `r format_sci(e, digits = 5, size = "small")`
- `r format_sci(e, digits = 4, size = "small")`
- `r format_sci(e, digits = 3, size = "small")`






## Formats {#formats}

The `format` argument appears in `format_numbers()` only. The default is "engr". The format is preset in the `format_dcml()`, `format_engr()`, and `format_sci()` convenience functions. 

To compare the effects across many orders of magnitude, we display the same vector in different formats.

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# 14. Comparing formats
x <- c(2.3333e-5, 3.4444e-4, 5.2222e-2, 6.3333e-1, 8.1111e+1, 9.2222e+2, 2.4444e+4, 3.1111e+5, 4.2222e+6)
dcml <- format_numbers(x, 3, format = "dcml")
sci <- format_numbers(x, 3, format = "sci")
engr <- format_numbers(x, 3, format = "engr")
DT <- data.table(dcml, sci, engr)
knitr::kable(DT,
  align = "r",
  col.names = c("decimal", "scientific", "engineering"),
  caption = "Example 14."
)
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```

The values displayed without powers-of-ten notation in the scientific and engineering columns are determined by the `omit_power` argument discussed next.  

## Excluding a range of exponents {#excluding-exponents}

When specifying power-of-ten notation, numbers with exponents lying within the range of the `omit_power` argument are typeset in decimal form. In engineering notation, the exponent is checked for lying within the range before and after the conversion to multiple-of-3 exponents.  

To illustrate, we compare two `omit_power` settings in both scientific and engineering formats. In some columns, we set `omit_power = NULL`, which imposes power-of-ten notation on the entire vector. 

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# 15. Effects of omit_power
DT <- atmos[3:12, .(pres)]
DT[, sci_all := format_sci(pres, 3, omit_power = NULL)]
DT[, sci_omit := format_sci(pres, 3, omit_power = c(-1, 0))]
DT[, engr_all := format_engr(pres, 3, omit_power = NULL)]
DT[, engr_omit := format_engr(pres, 3, omit_power = c(-1, 0))]
knitr::kable(DT,
  align = "r",
  col.names = c(
    "Unformatted",
    "all scientific",
    "scientific w/ omit",
    "all engineering",
    "engineering w/ omit"
  ),
  caption = "Example 15."
)
```

Comments: 

- In the "scientific with omit" column, the three values $\small (0.184, 1.05, 5.22)$ are in decimal form because their exponents lie within the `omit_power` range $\small \{-1, 0\}$. 
- The same is true in the "engineering with omit" column because these exponents are in the omit range  *before* converting to engineering form. 
- The "engineering with omit" column also displays the values $(\small 22, 80, 287)$ in decimal form because their exponents are in the omit range *after* converting to engineering notation. 

<br>

If a single value is assigned, e.g., `omit_power = 0`, the argument is interpreted as `c(0, 0)`.

```{r}
# 16. Omit power used for a single value of exponent
DT <- atmos[3:12, .(pres)]
DT[, sci_all := format_sci(pres, 3, omit_power = NULL)]
DT[, sci_omit := format_sci(pres, 3, omit_power = 0)]
DT[, engr_all := format_engr(pres, 3, omit_power = NULL)]
DT[, engr_omit := format_engr(pres, 3, omit_power = 0)]
knitr::kable(DT,
  align = "r",
  col.names = c(
    "Unformatted",
    "all scientific",
    "scientific w/ omit",
    "all engineering",
    "engineering w/ omit"
  ),
  caption = "Example 16."
)
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```

<br>

Setting `omit_power = c(-Inf, Inf)` yields the same decimal result as `format = "dcml"` and overrides any other `format` setting. For example, 

```{r}
# 17. Different ways of creating a decimal format
(y <- 6.78e-3)

(p <- format_numbers(y, 3, "sci", omit_power = c(-Inf, Inf)))

(q <- format_numbers(y, 3, "dcml"))

(r <- format_dcml(y, 3))

all.equal(p, q)
all.equal(p, r)
```

Example 17 (all cases) renders as, 

- `r format_numbers(y, 3, "dcml", size = "small")`



## Enforcing a specific exponent {#enforcing-exponent}

When values in a table column span only a few orders of magnitude, an audience is sometimes better served by setting the notation to a constant power of ten. For example, here we show numbers in scientific format and compare to columns in which the exponents are set to fixed values. Assigning a value to `set_power` overrides `omit_power` and  `format`. 

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# 18. set_power argument
DT <- atmos[alt <= 40, .(alt, pres, dens)]
DT[, sci_pres := format_sci(pres, 3, omit_power = c(-1, 2))]
DT[, set_pres := format_sci(pres, 3, omit_power = c(-1, 2), set_power = 3)]
DT[, sci_dens := format_engr(dens, 3, omit_power = c(-1, 2))]
DT[, set_dens := format_engr(dens, 3, omit_power = c(-1, 2), set_power = -2)]
DT[, pres := NULL]
DT[, dens := NULL]
knitr::kable(DT,
  align = "r",
  col.names = c("Altitude (km)", "Pressure (Pa)", "with set_power", "Density (kg/m$^{3}$)", "with set_power"),
  caption = "Example 18."
)
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```



## Options

Arguments assigned using `formatdown_options()` are described in the [Global settings](global_settings.html) article. 



## References

---
title: "formatdown"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{formatdown}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Please see the formatdown website for full documentation:

  * <https://graphdr.github.io/formatdown/index.html>
  
The website also includes articles describing typical applications in detail: 

  * [Index of articles](https://graphdr.github.io/formatdown/articles/index.html)
  

---
title: "Global settings"
vignette: >
  %\VignetteIndexEntry{Global settings}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output: rmarkdown::html_vignette
bibliography: ../inst/REFERENCES.bib
csl: ../inst/information-science-and-technology.csl
link-citations: yes
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  tidy = TRUE
)

# to knit "child" Rmd files
knitr::opts_knit$set(root.dir = "../")

library(formatdown)
library(data.table)
library(knitr)

options(
  datatable.print.nrows = 15,
  datatable.print.topn = 3,
  datatable.print.class = TRUE
)
```

![](../man/figures/options-img.png){width=70%}    
<small>*option* by Mike Lawrence is licensed under CC BY 2.0 DEED <https://creativecommons.org/licenses/by/2.0/>.</small>

<br>

Global options are provided for arguments that users are likely prefer to set once in a document instead of repeating in every function call. For example, some users prefer a comma decimal marker (",") throughout a document. 

Globally-set arguments can be overridden locally by assigning them in a function call.

## `formatdown_options()`

Set, examine, or reset several global options which affect the way in which a formatted object is rendered in an R markdown document. The options and their default settings are

```r
formatdown_options(delim = "$",
                   size = NULL,
                   decimal_mark = ".",
                   big_mark = "",
                   big_interval = 3,
                   small_mark = "",
                   small_interval = 5,
                   whitespace = "\\\\>", 
                   reset = FALSE)
```

To reset all formatdown arguments to their default values: 

```r
    formatdown_options(reset = TRUE)
```



<br>

*Usage.* &nbsp; For example, get two of the current settings.

```{r}
formatdown_options("size", "decimal_mark")
```

Assign new settings; examine result. 

```{r}
# Set
formatdown_options(size = "large", decimal_mark = ",")

# Examine result
formatdown_options("size", "decimal_mark")
```

Reset to default values; examine result.

```{r}
# Set to defaults
formatdown_options(reset = TRUE)

# Examine result
formatdown_options("size", "decimal_mark")
```


## Delimiters

Delimiters are characters that surround a formatted expression such that R Markdown renders it as an inline math expression. 

Sometimes the default `$ ... $` delimiters fail to render correctly. I encountered this once using `kableExtra::kbl()` in a `.qmd` document. The solution, suggested by the MathJax consortium [@Cervone:2018], is to use the delimiter pair `\( ... \)`, hence the built-in alternate, `delim = "\\("`. 

Left and right custom delimiters can be assigned in a vector, e.g., `c("\\[", "\\]")`.

<br>

*Examples.* &nbsp; Note that using `format_text()` introduces additional markup inside the delimiters. Details are described in the [Format text](format_text.html) article. 

```{r}
x <- 101300
txt <- "Hello world!"

# 1. Numeric input, default delimiters
format_dcml(x)

# 2. Numeric input, alternate delimiters
format_dcml(x, delim = "\\(")

# 3. Character input, default delimiters
format_text(txt)

# 4. Character input, alternate delimiters
format_text(txt, delim = "\\(")
```

<br>

Examples 1--4 (in inline code chunks) render as, 

1. `r format_dcml(x, size = "small")`

1. `r format_dcml(x, delim = "\\(", size = "small")`

1. `r format_text(txt, size = "small")`

1. `r format_text(txt, delim = "\\(", size = "small")`

<br>


## Font size

Font size is set using LaTeX-style macros inside the math-delimited expression. For example, with `size = "small"` (or `size = "\\small"`), the formatdown markup of the Avogadro constant would be  

        "$\\small 6.0221 \\times 10^{23}$", 

where the extra backslashes are necessary to escape the backslashes in `\small` and `\times`.  If `size = NULL` (default), no size command is added and the font size is equivalent to `"normalsize"`. 



<br>

*Examples.*

```{r}
# 5. Numeric input
format_dcml(x, size = "scriptsize")

# 6. Numeric input
format_dcml(x, size = "small")

# 7. Power-of-ten number using LaTeX-style size markup
format_sci(6.0221e+23, size = "\\small")

# 8. Character input, default size
format_text(txt)

# 9. Character input
format_text(txt, size = "large")
```

<br>

Examples 5--9 render as, 

5. `r format_dcml(x, size = "scriptsize")`

5. `r format_dcml(x, size = "small")`

5. `r format_sci(6.0221e+23, size = "small")`

5. `r format_text(txt)`

5. `r format_text(txt, size = "large")`

<br>


*Available sizes*

Comparing decimal notation, scientific notation, and text in possible font sizes (formatdown does not support the sizes: tiny, footnotesize, Large, LARGE, and Huge).  

```{r}
#| echo: false

x <- pi
y <- 5.3e+3
z <- "The cat"

scriptsize <- c(
  format_dcml(x, 5, size = "scriptsize"),
  format_sci(y, 1, size = "scriptsize"),
  format_text(z, size = "scriptsize")
)
small <- c(
  format_dcml(x, 5, size = "small"),
  format_sci(y, 1, size = "small"),
  format_text(z, size = "small")
)
normalsize <- c(
  format_dcml(x, 5, size = "normalsize"),
  format_sci(y, 1, size = "normalsize"),
  format_text(z, size = "normalsize")
)
large <- c(
  format_dcml(x, 5, size = "large"),
  format_sci(y, 1, size = "large"),
  format_text(z, size = "large")
)
huge <- c(
  format_dcml(x, 5, size = "huge"),
  format_sci(y, 1, size = "huge"),
  format_text(z, size = "huge")
)

DT <- data.table(scriptsize, small, normalsize, large, huge)
knitr::kable(DT, align = "r")
```



## Decimal separator

For a number written in decimal form, the decimal mark separates the integer part from the fractional part. 

- A period or dot (".") is the conventional decimal mark in the US, Australia, Canada (English-speaking), Mexico, the UK, much of eastern Asia, and other regions. 

- A comma (",") is the conventional decimal mark in Brazil, Canada (French-speaking), much of Europe and Latin America, Russia, and other regions. 

The decimal mark in formatdown may be reset locally in a function call or globally using `formatdown_options()`; it is not affected by the base R option `OutDec`. 

<br>

*Examples.*

```{r}
# 10. Decimal markup
x <- pi
format_dcml(x, 5, decimal_mark = ",")

# 11. Power-of-ten markup
y <- 1.602176634e-19
format_sci(y, 5, decimal_mark = ",")
```


<br>

Examples 10 and 11 render as, 

10. `r format_dcml(x, 5, decimal_mark = ",", size = "small")`

10. `r format_sci(y, 5, decimal_mark = ",", size = "small")`

<br>



## Separating digits

The NIST recommends we use a thin space to separate more than 4 digits to the left or to the right of a decimal marker [@NIST:2022, 10.5.3]:

> ... digits should be separated into groups of three, counting from the decimal marker towards the left and right, by the use of a thin, fixed space. However, this practice is not usually followed for numbers having only four digits on either side of the decimal marker except when uniformity in a table is desired.

Both `big_mark` and `small_mark` add the horizontal-space characters inside the math delimiters; `big_mark` to the integer portion and `small_mark` to the fractional portion. The possible values are empty `""` (default), `"thin"`, or the thin-space macro itself `\\\\,`. 

The interval arguments `big_interval` and `small_interval` set the number of digits separated by thin spaces when `big_mark` or `small_mark` are not empty. However, formatdown does not encode the exemption for 4-digit groups mentioned in the NIST quote above.  

<br>


*Examples.*

```{r}
w <- 1013
x <- 101300
y <- 0.002456
z <- x + y

# 12. 4-digit number, no space
format_dcml(w)

# 13. 4-digit number, with space
format_dcml(w, big_mark = "thin")

# 14. Group digits to the left of the decimal
format_dcml(x, big_mark = "thin")

# 15. Group digits to the right of the decimal
format_dcml(y, small_mark = "\\\\,")

# 16. Change the small interval
format_dcml(y, small_mark = "\\\\,", small_interval = 3)

# 17. Group digits to the left and right of the decimal
format_dcml(z, 12, big_mark = "thin", small_mark = "thin")
```


<br>

Examples 12--17 render as, 

12. `r format_dcml(w, size = "small")`

12. `r format_dcml(w, big_mark = "thin", size = "small")`

12. `r format_dcml(x, big_mark = "\\\\,", size = "small")`

12. `r format_dcml(y, small_mark = "\\\\,", size = "small")`

12. `r format_dcml(y, small_mark = "\\\\,", small_interval = 3, size = "small")`

12. `r format_dcml(z, 12, big_mark = "thin", small_mark = "\\\\,", size = "small")`



## Preserving text spaces

The horizontal-space macro is used to preserve spaces in text formatted with `format_text()` as well as spaces within physical-unit strings with `format_numbers()`. Without it, an inline math markup such as 

        $\mathrm{This Is Math Text.}$

is rendered in an R markdown document as 

$\qquad$ `r "$\\small\\mathrm{This Is Math Text.}$"`

To preserve such spaces, formatdown substitutes the character string  `\>` for each space, producing output like the following, where backslashes have been escaped, 

        "$\\mathrm{This\\>Is\\>Math\\>Text.}$"

rendered as, 

$\qquad$ `r "$\\small\\mathrm{This\\>Is\\>Math\\>Text.}$"`


Because the backslashes must be escaped, the formatdown output is $\small\mathtt{\verb|"\\>"|}$, but the the argument value set by the user is  `whitespace =` $\small\mathtt{\verb|"\\\\>"|}$. One may also use $\mathtt{\small\verb|"\\\\:"|}$ or $\mathtt{\small\verb|"\\\\ "|}$. 


<br>

*Examples.*

```{r}
# 18. Character input, default space "\>"
format_text(txt, whitespace = "\\\\>")

# 19. Character input, alternate space "\:"
format_text(txt, whitespace = "\\\\:")

# 20. Character input, alternate space "\ "
format_text(txt, whitespace = "\\\\ ")
```

<br>

Examples 18--20 render as, 

18. `r format_text(txt, size = "small")` 

18. `r format_text(txt, whitespace = "\\\\:", size = "small")`

18. `r format_text(txt, whitespace = "\\\\ ", size = "small")` 







## Applications


*Example 21.*

In this example, we format different columns of a data frame using `decimal_mark`, `big_mark` and `small_mark`. 

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# Set options
formatdown_options(decimal_mark = ",", big_mark = "thin", small_mark = "thin")

# Use water data included with formatdown
DT <- copy(water)[1:6]

# Examine the data frame
DT[]

# Routine decimal formatting
DT$temp <- format_dcml(DT$temp)
DT$dens <- format_dcml(DT$dens)

# Omit big_mark spacing for 4 digits
DT$sp_wt <- format_dcml(DT$sp_wt, 4, big_mark = "")

# Set significant digits (viscosity) to achieve a consistent string length
DT[visc >= 0.001, temp_visc := format_dcml(visc, 5)]
DT[visc < 0.001, temp_visc := format_dcml(visc, 4)]
DT[, visc := temp_visc]
DT[, temp_visc := NULL]

# Will appear with big_mark spacing, change from Pa to kPa
DT$bulk_mod_kPa <- format_dcml(DT$bulk_mod / 1000, 4)
DT$bulk_mod <- NULL

knitr::kable(DT, align = "r", caption = "Example 21.")

# Set package options to default values
formatdown_options(reset = TRUE)
```

<br>

*Example 22.*

Same table, but using power of ten formatting. 

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# Use water data included with formatdown
DT <- copy(water)[1:6]

# Routine decimal formatting
cols <- c("temp", "dens", "sp_wt")
DT[, (cols) := lapply(.SD, function(x) format_dcml(x)), .SDcols = cols]

# Power of ten
DT$bulk_mod <- format_engr(DT$bulk_mod, 3)
DT$visc <- format_engr(DT$visc, 4, set_power = -3)

knitr::kable(DT, align = "r", caption = "Example 22.")
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```

<br>

*Example 23.*

The `metals` data set includes columns of text and  decimal and power-of-ten numbers. 

```{r}
#| echo: false
formatdown_options(size = "small")
```

```{r}
# Use water data included with formatdown
DT <- copy(metals)

# Examine the data frame
DT[]

# Text
DT$metal <- format_text(DT$metal)

# Decimal
cols <- c("dens", "thrm_cond")
DT[, (cols) := lapply(.SD, function(x) format_dcml(x)), .SDcols = cols]

# Power of ten
cols <- c("elast_mod", "thrm_exp")
DT[, (cols) := lapply(.SD, function(x) format_engr(x, 3)), .SDcols = cols]

knitr::kable(DT, align = "lrrrr", caption = "Example 23.")
```

```{r}
#| echo: false
formatdown_options(reset = TRUE)
```





## References

---
title: "Format text"
vignette: >
  %\VignetteIndexEntry{Format text}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output: rmarkdown::html_vignette
bibliography: ../inst/REFERENCES.bib
csl: ../inst/information-science-and-technology.csl
link-citations: yes
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  tidy = TRUE
)

# to knit "child" Rmd files
knitr::opts_knit$set(root.dir = "../")

library(formatdown)
library(data.table)
library(knitr)

options(
  datatable.print.nrows = 15,
  datatable.print.topn = 3,
  datatable.print.class = TRUE
)
```

```{r}
#| echo: false

# make header table, but scan it and save as png

# Markup <- c("$\\small\\mathtt{\\verb+\\mathrm{...}+}$", "$\\small\\mathtt{\\verb+\\mathit{...}+}$", "$\\small\\mathtt{\\verb+\\mathbf{...}+}$", "$\\small\\mathtt{\\verb+\\mathsf{...}+}$", "$\\small\\mathtt{\\verb+\\mathtt{...}+}$")
#
# Style <- c("$\\small\\mathrm{plain}$", "$\\small\\mathit{italic}$", "$\\small\\mathbf{bold}$", "$\\small\\mathsf{sans\\ serif}$", "$\\small\\mathtt{typewriter}$")
#
# DT <- data.table(Markup, Style)
# knitr::kable(DT, align = "l") |>
#   kableExtra::kable_styling(font_size = 16, position = "left") |>
#   kableExtra::column_spec(1:2, width = "2in")
```

![](../man/figures/latex_math_formatting_text_table_2.png){width=70%}    

<br>

Columns of text in a data table will generally not be rendered in the same typeface as produced by `format_numbers()`. The `format_text()` function provides an approach to assigning a matching typeface to such columns.  


<br>

*Packages.* &nbsp; If you are writing your own script to follow along, we use the following packages in this vignette. Data frame operations are performed with data.table syntax. Some users may wish to translate the examples to use  base R or dplyr syntax.  

```{r}
#| echo: true
#| eval: false

library("formatdown")
library("data.table")
library("knitr")
```

## Markup

We format values as inline math expressions delimited by `$ ... $` or the optional `\( ... \)`. Inside the math delimiters, the text is formatted using math-text macros such as `\mathrm`, `\mathit`, etc. 

For example, the text "Hello world!" is marked up as follows, where the space between words is preserved by a horizontal space macro (`\>`), 

        $\mathrm{Hello\>world!}$

and is rendered in an R markdown document as: $\small\mathrm{Hello\>world!}$. To *program* the markup, however, we enclose the markup in quote marks as a character string, that is, 

        "$\\mathrm{Hello\\>world!}$"

where extra backslashes are necessary to "escape" the backslashes in `\mathrm` and `\>`. When the optional font size argument is assigned, formatdown adds a LaTeX-style sizing macro such as `\small` or `\large`, for example, 

        "$\\small\\mathrm{Hello\\>world!}$", 

where again the markup includes an extra backslash. 










## `format_text()`

Converts a character vector (or a vector coercible to character class) to math-text of the form, 

        "$\\math**{txt}$" 
 
where `txt` is the text to be formatted and  `\\math**` is a font face macro such as `\\mathrm`, `\\mathit`, etc. 

<br>

*Usage.* &nbsp; 

``` r
format_text(x,
            face = "plain",
            ...,
            size = formatdown_options("size"),
            delim = formatdown_options("delim"),
            whitespace = formatdown_options("whitespace"))
```

- Arguments before the dots do not have to be named if argument order is maintained. 
- Arguments after the dots (`...`) must be named. 
- The arguments assigned via `formatdown_options()` can be reset by the user  locally in a function call or globally using `formatdown_options()`. 
 


<br>

*Examples.*  &nbsp; These examples are shown with default arguments. Arguments are explored more fully starting with the next section. 

```{r}
# 1. One string
x <- "Alum 6061"
format_text(x)

# 2. String vector
y <- c("Alum 6061", "Carbon steel", "Ni-Cr-Fe alloy")
format_text(y)
```

Examples 1 and 2 (in inline code chunks) render as,    

1. `r format_text(x, size = "small")`
1. `r format_text(y, size = "small")`








## Input coercible to character

Illustrating that variables of different classes are coerced to character if possible. Starting with character class, 

```{r}
# 3. Character class
x <- c("abc", "def", NA_character_)
format_text(x)
```

Example 3 renders as: &nbsp; `r format_text(x, size = "small")`.  

```{r}
# 4. Numeric class
x <- c(10, 3E-5, 4.56E+10)
format_text(x)
```

Example 4 renders as: &nbsp; `r format_text(x, size = "small")`. 

```{r}
# 5. Logical class
x <- TRUE
format_text(x)
```

Example 5 renders as: &nbsp; `r format_text(x, size = "small")`. 

```{r}
# 6. Complex class
x <- 2 + 3i
format_text(x)
```

Example 6 renders as: &nbsp; `r format_text(x, size = "small")`. 

```{r}
# 7. Date class
x <- Sys.Date()
format_text(x)
```

Example 7 renders as: &nbsp; `r format_text(x, size = "small")`. 

```{r}
# 8 Factor class
x <- as.factor(c("low", "med", "high"))
format_text(x)
```

Example 8 renders as: &nbsp; `r format_text(x, size = "small")`.  

```{r}
# 9. NULL class
x <- NULL
format_text(x)
```

Example 9 renders as a zero-length character (no visible output rendered). 





## Typeface

Format the same column of text using each of the five possible `face` arguments for comparison. 

```{r}
# 10. Compare available typefaces
x <- c("One day", "at a", "time.")
plain <- format_text(x, face = "plain", size = "small")
italic <- format_text(x, face = "italic", size = "small")
bold <- format_text(x, face = "bold", size = "small")
sans <- format_text(x, face = "sans", size = "small")
mono <- format_text(x, face = "mono", size = "small")
DT <- data.table(plain, italic, bold, sans, mono)
knitr::kable(DT, align = "l", caption = "Example 10.")
```

One may use the LaTeX-style macro instead of the shorthand argument option. For example, 

```{r}
# 11. Two equivalent values for argument
hello_text <- "Hello world!"
p <- format_text(hello_text, face = "plain")
q <- format_text(hello_text, face = "\\mathrm")

# Demonstrate equivalence
all.equal(p, q)
```

Example 11 renders as:

- From *p*: `r format_text(hello_text, face = "plain", size = "small")`
- From *q*: `r format_text(hello_text, face = "\\mathrm", size = "small")`



## Special characters in math mode

The argument of `format_text()` is evaluated within a math-markup. Thus any math syntax in your text, such as an underscore "_" or caret "^", are rendered in math mode, not verbatim. For example, the underscore creates a subscript and the caret creates a superscript, 

```{r}
# 12. Special characters NOT escaped
format_text("R_e")
format_text("m^3")
```

Example 12 renders as:

- `r format_text("R_e", size = "small")` 
- `r format_text("m^3", size = "small")`

If we wanted to retain the underscore or caret as characters, we can try to escape the special character or use the LaTeX verbatim function,  

```{r}
# 13. Special characters escaped
format_text("R\\_e")
format_text("m\\verb+^+3")
```

Example 13 renders as:

- `r format_text("R\\_e", size = "small")`
- `r format_text("m\\verb+^+3", size = "small")`

## Options

Arguments assigned using `formatdown_options()` are described in the [Global settings](global_settings.html) article. 

---
title: "convert hyphenated isotopic data to nuclear notation"
format: docx
editor: visual
editor_options: 
  chunk_output_type: console
---

Use this script in Rstudio to convert isotopic data from the hyphenated notation, e.g. Ba-137m, to nuclear notation, e.g. $^{137m}Ba$.

First you need to load your data. Modify the example below to identify your data by the correct name and file location (path).

Your data will only be converted after rendering. A Word document will be created in the directory where you have this script.

```{r}
#| label: libraries
#| echo: false
#| message: false
#| warning: false
#| fig-width: 8

library(formatdown)
library(dplyr)
library(stringr)
library(knitr)
```

```{r}
#| label: isotopes
#| echo: false
#| message: false
#| warning: false
#| fig-width: 8

# Load your data, and call it "data" or modify everything below to match.
# The script assumes your data columns are called "Nuclide" and "Activity". 
# You can modify this script to only use this chunk or the next chunk, depending on whether you have numbers to convert, activity to convert, or both.
# 
#  Change the example data to load your data.
#  
data <- data.frame("Nuclide" = c("Nb-93m", "Ni-63", "Cd-113", "Th-234", 
"Ag-108m", "Y-90", "Cm-245", "Th-228", "Th-229", "Cm-248"))

# Locate dash in RN's
df <- data.frame("Nuclide" = data$Nuclide)

pos_dash <- str_locate(df$Nuclide, "-")[, 1]
RN_length <- str_length(df$Nuclide)

# X is the element symbol, e.g. Ba
# A is the isotope number, including, if needed m for metastable, e.g. 137m

df$X <- str_sub(df$Nuclide, start = 1, end = pos_dash - 1)
df$A <- str_sub(df$Nuclide, start = pos_dash + 1, end = RN_length)

isotope <- paste0("$", "^{", df$A, "}", df$X, "$")

knitr::kable(isotope, row.names = FALSE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatdown-package.R
\docType{package}
\name{formatdown-package}
\alias{formatdown}
\alias{formatdown-package}
\title{formatdown: Formatting Numbers in 'rmarkdown' Documents}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides a small set of tools for formatting numbers in R-markdown documents. Convert a numerical vector to character strings in power-of-ten form, decimal form, or measurement-units form; convert symbols for chemical isotopes from hyphenated form to nuclear form; all are math-delimited for rendering as inline equations. Can also convert text into math-delimited text to match the font face and size of math-delimited numbers. Useful for rendering single numbers in inline R code chunks and for rendering columns in tables.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/graphdr/formatdown/}
  \item \url{https://graphdr.github.io/formatdown/}
  \item \url{https://CRAN.R-project.org/package=formatdown}
  \item Report bugs at \url{https://github.com/graphdr/formatdown/issues}
}

}
\author{
\strong{Maintainer}: Richard Layton \email{graphdoctor@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{air_meas}
\alias{air_meas}
\title{Air density measurements}
\format{
Classes data.table and data.frame with	5 observations
of 7 variables:

\describe{
\item{date}{"Date" class format "YYYY-MM-DD".}
\item{trial}{Character, label "a" through "e".}
\item{humid}{Factor, humidity, "low", "med", or "high."}
\item{temp}{Numeric, measured temperature (K).}
\item{pres}{Numeric, measured atmospheric pressure (Pa).}
\item{sp_gas}{Numeric, specific gas constant in mass form \eqn{R_{sp}},
ideal gas reference value, (J kg\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}}).}
\item{dens}{Numeric, calculated air density
\eqn{\rho} = \eqn{p}\eqn{R_{sp}^{-1}}{^{-1}}\eqn{T^{-1}}{^{-1}}
(kg m\eqn{^{-3}}{^{-3}}).} }
}
\usage{
data(air_meas, package = "formatdown")
}
\description{
Table of air properties at room temperature and pressure, simulating multiple
measurements at approximately steady state,
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatdown_options.R
\name{formatdown_options}
\alias{formatdown_options}
\title{Get and set function arguments via options}
\usage{
formatdown_options(..., reset = FALSE)
}
\arguments{
\item{...}{One or more \code{name = value} pairs to set values; or one or more
quoted option names to get values.}

\item{reset}{Logical vector of length 1; if TRUE, reset all options to their
default values.}
}
\value{
Nothing; used for its side-effect.
}
\description{
Changes the default values of function arguments which affect the markup and
appearance of formatdown results.
}
\details{
Global options are provided for arguments that users would likely prefer to
set once in a document instead of repeating in every function call. For
example, some users prefer a comma decimal marker (",") throughout a
document.

Globally-set arguments can be overridden locally by assigning them in a
function call.

The arguments that can be set with this function are as follows:
\itemize{
\item \code{delim}:          Character, length 1 or 2, to define the left and right math
markup delimiters. The default setting, \code{delim = "$"}, produces
left and right delimiters \verb{$...$}. The alternate built-in
setting, \code{delim = "\\\\("}, produces left and right delimiters
\verb{\\\\( ... \\\\)}. Custom delimiters can be assigned in a vector of
length 2 with left and right delimiter symbols, e.g.,
\verb{c("\\\[", "\\\]")}. Special characters typically must be escaped.
\item \code{size}:           Character, length 1, to assign a font size. If not empty, adds
a font size macro to the markup inside the math delimiters. Possible
values are \code{"scriptsize"}, \code{"small"}, \code{"normalsize"}, \code{"large"}, and
\code{"huge"}. One may also assign the equivalent LaTeX-style markup itself,
e.g., \code{"\\\\scriptsize"}, \code{"\\\\small"}, etc. Default is NULL.
\item \code{decimal_mark}:   Character, length 1, to assign the decimal marker.
Possible values are a period \code{"."} (default) or a comma \code{","}. Passed
to \code{formatC(decimal.mark)}.
\item \code{big_mark}:       Character, length 1, used as the mark between every
\code{big_interval} number of digits to the left of the decimal marker to
improve readability. Possible values are empty \code{""} (default) or
\code{"thin"} to produce a LaTeX-style thin, horizontal space. One may also
assign the thin-space markup itself \code{"\\\\\\\\,"}.
Passed to \code{formatC(big.mark)}.
\item \code{big_interval}:   Integer, length 1, that defines the number of digits
(default 3) in groups separated by \code{big_mark}. Passed to
\code{formatC(big.interval)}.
\item \code{small_mark}:     Character, length 1, used as the mark between every
\code{small_interval} number of digits to the right of the decimal marker to
improve readability. Possible values are empty \code{""} (default) or
\code{"thin"} to produce a LaTeX-style thin, horizontal space. One may also
assign the thin-space markup itself \code{"\\\\\\\\,"}.
Passed to \code{formatC(small.mark)}.
\item \code{small_interval}: Integer, length 1, that defines the number of digits
(default 5) in groups separated by \code{small_mark}. Passed to
\code{formatC(small.interval)}.
\item \code{whitespace}:     Character, length 1, to define the LaTeX-style
math-mode macro to preserve a horizontal space between words of text or
between physical-unit abbreviations when formatting numbers of class
"units". Default is \code{"\\\\\\\\ "}. Alternatives include \code{"\\\\\\\\:"} or
"\verb{\\\\\\\\>}".
\item \code{multiply_mark}:  Character, length 1, to define the multiplication symbol
in power of ten notation. Possible values are \code{"\\\\times"} (default)
or a half-high dot \code{"\\\\cdot"} which is often used when the decimal
mark is a comma.
\item \code{Z}:              Logical, length 1, default FALSE, used to determine whether or
not the atomic number subscript is used in formatting nuclear
notation of isotopes.
\item \code{warn}:           Logical, length 1, default TRUE, used to determine whether or
not a warning is issued when an input for a chemical element in
hyphenated notation contains an error in the element symbol or the
mass number of the isotope.
}
}
\examples{
# Show all options
formatdown_options()

# Store existing settings, including any changes made by the user
old_settings <- formatdown_options()

# View one option
formatdown_options()$delim

# View multiple options
formatdown_options("size", "delim")

# Change options
formatdown_options(size = "small", delim = "\\\\(")
formatdown_options("size", "delim")

# Reset to default values
formatdown_options(reset = TRUE)
formatdown_options("size", "delim")

# Reset options to those before this example was run
do.call(formatdown_options, old_settings)

# Option effects

# delim
x <- 101300
format_dcml(x)
# equivalent to
format_dcml(x, delim = c("$", "$"))
# built-in alternate
format_dcml(x, delim = "\\\\(")
# equivalent to
format_dcml(x, delim = c("\\\\(", "\\\\)"))

# size
format_dcml(x, size = "small")
# equivalent to
format_dcml(x, size = "\\\\small")
# other possible values
format_dcml(x, size = "scriptsize")
format_dcml(x, size = "large")
format_dcml(x, size = "huge")
# default NULL
format_dcml(x, size = NULL)
# renders equivalent to
format_dcml(x, size = "normalsize")

# decimal_mark
y <- 6.02214076E+10
format_sci(y, 5, decimal_mark = ".")
format_sci(y, 5, decimal_mark = ",")

# big_mark
format_dcml(y, 9)
format_dcml(y, 9, big_mark = "thin")
# equivalent to
format_dcml(y, 9, big_mark = "\\\\\\\\,")

# big_interval
format_dcml(y, 9, big_mark = "thin", big_interval = 3)
format_dcml(y, 9, big_mark = "thin", big_interval = 5)

# small_mark
z <- 1.602176634e-8
format_sci(z, 10)
format_sci(z, 10, small_mark = "thin")
format_sci(z, 10, small_mark = "\\\\\\\\,")
format_engr(z, 10, small_mark = "thin")

# small_interval
format_sci(z, 10, small_mark = "thin", small_interval = 3)
format_sci(z, 10, small_mark = "thin", small_interval = 5)
format_engr(z, 10, small_mark = "thin", small_interval = 5)

# whitespace in text
p <- "Hello world!"
format_text(p)
# equivalent to
format_text(p, whitespace = "\\\\\\\\ ")
# alternates
format_text(p, whitespace = "\\\\\\\\:")
format_text(p, whitespace = "\\\\\\\\>")

# whitespace in physical units expression
x <- pi
units(x) <- "m/s"
format_dcml(x)
# equivalent to
format_dcml(x, whitespace = "\\\\\\\\ ")

# multiply_mark
x <- 101300
format_engr(x, decimal_mark = ".", multiply_mark = "\\\\times")
format_engr(x, decimal_mark = ",", multiply_mark = "\\\\cdot")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{element_set}
\alias{element_set}
\title{Chemical elements}
\format{
Classes data.table and data.frame with	354
observations of 4 variables:

\describe{
\item{element}{Character, element name, hydrogen (atomic number 1)
through oganesson (atomic number 118).}
\item{symbol}{Character, element symbol.}
\item{atomic_number}{Character, 1-118.}
\item{mass_number}{Character, 1-294.}
}
}
\usage{
data(element_set, package = "formatdown")
}
\description{
Table of 118 chemical elements with name, symbol, atomic number, and
mass numbers. Elements with more than one possible mass number have
separate rows for each isotope.
}
\details{
In "AZE" or "AZX" notation, "A" is the mass number, "Z" is the atomic number,
and "E" or "X" is the element symbol.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{atmos}
\alias{atmos}
\title{Properties of standard atmosphere}
\format{
Classes data.table and data.frame with 20 observations
of 5 variables:

\describe{
\item{alt}{Numeric, altitude (km)}
\item{temp}{Numeric, air temperature (K)}
\item{pres}{Numeric, atmospheric pressure (Pa)}
\item{dens}{Numeric, air density (kg m\eqn{^{-3}}{^{-13}})}
\item{sound}{Numeric, speed of sound (m/s)}
}
}
\source{
\emph{Marks' Standard Handbook for Mechanical Engineers 9/e} (1987)
E.A. Avallone and T. Baumeister (ed.),
"Table 4.2.2 International Standard Atmosphere",
pp. 4-38,  McGraw-Hill, NY.
}
\usage{
data(atmos, package = "formatdown")
}
\description{
Table of atmospheric properties as a function of altitude, sea level to
1000 km.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{metals}
\alias{metals}
\title{Properties of metals}
\format{
Classes data.table and data.frame with	6 observations
of 5 variables:

\describe{
\item{metal}{Character, name of material}
\item{dens}{Numeric, density (kg m\eqn{^{-3}}{^{-3}})}
\item{thrm_exp}{Numeric, coefficient of thermal expansion
(m m\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}})}
\item{thrm_cond}{Numeric, thermal conductivity (W m\eqn{^{-1}}{^{-1}}K\eqn{^{-1}}{^{-1}})}
\item{elast_mod}{Numeric, modulus of elasticity (Pa)}
}
}
\source{
\emph{Marks' Standard Handbook for Mechanical Engineers 9/e} (1987)
E.A. Avallone and T. Baumeister (ed.),
"Basic Properties of Several Metals",
pp. 6-11,  McGraw-Hill, NY.
}
\usage{
data(metals, package = "formatdown")
}
\description{
Table of mechanical and thermal properties of selected metals.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_text.R
\name{format_text}
\alias{format_text}
\title{Format text}
\usage{
format_text(
  x,
  face = "plain",
  ...,
  size = formatdown_options("size"),
  delim = formatdown_options("delim"),
  whitespace = formatdown_options("whitespace")
)
}
\arguments{
\item{x}{Text to be formatted. Can be a single string,
a vector, or a column of a data frame.}

\item{face}{Font face. Determines the font face macro inside the math
delimiters. Possible values are \code{"plain"} (default), \code{"italic"},
\code{"bold"}, \code{"sans"}, or \code{"mono"}. One may assign instead the
corresponding LaTeX-style markup itself, e.g., \verb{\\mathrm},
\verb{\\mathit}, \verb{\\mathbf}, \verb{\\mathsf}, or \verb{\\mathtt}.}

\item{...}{Not used for values; forces subsequent arguments to be
referable only by name.}

\item{size, delim, whitespace}{Used to format the math-delimited character
strings. For details, see the help page for \code{formatdown_options()}.}
}
\value{
A character vector with elements delimited as inline math markup
in plain, italic, sans serif, bold, or monospace font face.
}
\description{
Convert a character vector to "math text" delimited for rendering as inline
equations in an R markdown document. Particularly useful for matching the
font face of character columns to that of numerical columns in a table.
}
\details{
Given a scalar, vector, or data frame column, \code{format_text()} converts its
argument to a character string of the form \code{"$\\\\mathxx{a}$"} where \code{a}
is the element to be formatted and \verb{\\\\mathxx} determines the font face:
plain type is set by \verb{\\\\mathrm}; italic by \verb{\\\\mathit};
bold by \verb{\\\\mathbf}; sans serif by \verb{\\\\mathsf}; and monospace (typewriter
text) by \verb{\\\\mathtt}. All strings include markup delimiters \verb{$...$} for
rendering (in an R markdown or Quarto markdown document) as an inline
equation.
}
\examples{
# Text vector

# default face = "plain"
x <- air_meas$humid
format_text(x)

# equivalently
format_text(x, face = "plain")

# input vector
x <- c("Hello world!", "Goodbye blues!")
format_text(x)

# argument coerced to character string if possible
format_text(c(1.2, 2.3, 3.4))
format_text(x = NA)
format_text(x = c(TRUE, FALSE, TRUE))

# numbers as strings are rendered as-is
format_text(x = c("1.2E-3", "3.4E+0", "5.6E+3"))

# other font faces
format_text(x, face = "italic")
format_text(x, face = "bold")
format_text(x, face = "sans")
format_text(x, face = "mono")
}
\seealso{
Other format_*: 
\code{\link{format_dcml}()},
\code{\link{format_engr}()},
\code{\link{format_nucl}()},
\code{\link{format_numbers}()},
\code{\link{format_sci}()}
}
\concept{format_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_nucl.R
\name{format_nucl}
\alias{format_nucl}
\title{Format isotopes in nuclear notation}
\usage{
format_nucl(
  x,
  face = "plain",
  ...,
  Z = formatdown_options("Z"),
  warn = formatdown_options("warn"),
  delim = formatdown_options("delim")
)
}
\arguments{
\item{x}{Character. Hyphenated form of chemical elements or isotopes.
Can be a single character string or a vector of strings. Must
include a single hyphen between the element symbol and the mass
number. For example, carbon 12 in hyphenated notation is the
character string \code{"C-12"}.}

\item{face}{Font face. Determines the font face macro inside the math
delimiters. Possible values are \code{"plain"} (default), \code{"italic"},
\code{"bold"}, \code{"sans"}, or \code{"mono"}. One may assign instead the
corresponding LaTeX-style markup itself, e.g., \verb{\\mathrm},
\verb{\\mathit}, \verb{\\mathbf}, \verb{\\mathsf}, or \verb{\\mathtt}.}

\item{...}{Not used for values; forces subsequent arguments to be
referable only by name.}

\item{Z}{T/F add the atomic number. For details,
see the help page for \code{formatdown_options()}.}

\item{warn}{T/F issue warning for incorrect input \code{x}. For details,
see the help page for \code{formatdown_options()}.}

\item{delim}{The math-delimiter. For details, see the help page for
\code{formatdown_options()}.}
}
\value{
A character vector of isotopes in nuclear notation with elements
delimited as inline math markup in plain, italic, sans serif,
bold, or monospace font face.
}
\description{
Convert chemical element or isotope from from hyphenated notation to
nuclear notation.
}
\details{
We start with a character string in hyphenated form \emph{X-A} where \emph{X} is the
chemical symbol of an element and \emph{A} is its mass number.
For example, carbon 12 would be written as the string \code{"C-12"}.

Given a character scalar, vector, or data frame column of isotopes in
this form, \code{format_nucl()} constructs the form \code{"$\\\\mathxx{^{A}X}$"} where
\verb{\\\\mathxx} determines the font face: plain type is set by \verb{\\\\mathrm}; italic
by \verb{\\\\mathit}; bold by \verb{\\\\mathbf}; sans serif by \verb{\\\\mathsf}; and monospace
(typewriter text) by \verb{\\\\mathtt}. The result includes markup delimiters \verb{$...$}
for rendering (in an R markdown or Quarto markdown document) as an inline
code chunk.

Unlike other functions in formatdown, \code{format_nucl()} does not support a
\code{size} argument.
}
\examples{
# default arguments
x <- "C-12"
format_nucl(x)

# add atomic number
format_nucl(x, Z = TRUE)

# vector of isotopes
x <- c("C-12", "Fe-54", "U-238")
format_nucl(x, Z = TRUE)

# turn off the warning when using symbols
x <- "X-A"
format_nucl(x, Z = TRUE, warn = FALSE)

# change the typeface
x <- "C-12"
format_nucl(x, face = "bold")

# obtain all possible mass numbers for an element
element_set[symbol == "Ca"]
}
\seealso{
Other format_*: 
\code{\link{format_dcml}()},
\code{\link{format_engr}()},
\code{\link{format_numbers}()},
\code{\link{format_sci}()},
\code{\link{format_text}()}
}
\concept{format_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_numbers.R
\name{format_engr}
\alias{format_engr}
\title{Format engineering notation}
\usage{
format_engr(
  x,
  digits = 4,
  ...,
  omit_power = c(-1, 2),
  set_power = NULL,
  delim = formatdown_options("delim"),
  size = formatdown_options("size"),
  decimal_mark = formatdown_options("decimal_mark"),
  small_mark = formatdown_options("small_mark"),
  small_interval = formatdown_options("small_interval"),
  whitespace = formatdown_options("whitespace"),
  multiply_mark = formatdown_options("multiply_mark")
)
}
\arguments{
\item{x}{Number or numbers to be formatted. Can be a single number, a vector,
or a column of a data frame.}

\item{digits}{Integer from 1 through 20 that controls the number of
significant digits in printed numeric values. Passed to \code{signif()}.
Default is 4.}

\item{...}{Not used for values; forces subsequent arguments to be
referable only by name.}

\item{omit_power}{Numeric vector \code{c(p, q)} with \code{p <= q}, specifying
the range of exponents over which power-of-ten notation is omitted in
either scientific or engineering format. Default is
\code{c(-1, 2)}. If a single value is assigned, i.e., \code{omit_power = p}, the
argument is interpreted as \code{c(p, p)}. If \code{NULL} or \code{NA}, all elements
are formatted in power-of-ten notation. Argument is overridden by
specifying \code{set_power} or decimal notation.}

\item{set_power}{Integer, length 1. Formats all values in \code{x} with the same
power-of-ten exponent. Default NULL. Overrides \code{format} and \code{omit_power}
arguments.}

\item{delim}{Character, length 1 or 2, to define the left and right math
markup delimiters. The default setting, \code{delim = "$"}, produces
left and right delimiters \verb{$...$}. The alternate built-in
setting, \code{delim = "\\\\("}, produces left and right delimiters
\verb{\\\\( ... \\\\)}. Custom delimiters can be assigned in a vector of
length 2 with left and right delimiter symbols, e.g.,
\verb{c("\\\[", "\\\]")}. Special characters typically must be escaped.}

\item{size}{Character, length 1, to assign a font size. If not empty, adds
a font size macro to the markup inside the math delimiters. Possible
values are \code{"scriptsize"}, \code{"small"}, \code{"normalsize"}, \code{"large"}, and
\code{"huge"}. One may also assign the equivalent LaTeX-style markup itself,
e.g., \code{"\\\\scriptsize"}, \code{"\\\\small"}, etc. Default is NULL.}

\item{decimal_mark}{Character, length 1, to assign the decimal marker.
Possible values are a period \code{"."} (default) or a comma \code{","}. Passed
to \code{formatC(decimal.mark)}.}

\item{small_mark}{Character, length 1, used as the mark between every
\code{small_interval} number of digits to the right of the decimal marker to
improve readability. Possible values are empty \code{""} (default) or
\code{"thin"} to produce a LaTeX-style thin, horizontal space. One may also
assign the thin-space markup itself \code{"\\\\\\\\,"}.
Passed to \code{formatC(small.mark)}.}

\item{small_interval}{Integer, length 1, that defines the number of digits
(default 5) in groups separated by \code{small_mark}. Passed to
\code{formatC(small.interval)}.}

\item{whitespace}{Character, length 1, to define the LaTeX-style
math-mode macro to preserve a horizontal space between words of text or
between physical-unit abbreviations when formatting numbers of class
"units". Default is \code{"\\\\\\\\ "}. Alternatives include \code{"\\\\\\\\:"} or
"\verb{\\\\\\\\>}".}

\item{multiply_mark}{Character, length 1, to define the multiplication symbol
in power of ten notation. Possible values are \code{"\\\\times"} (default)
or a half-high dot \code{"\\\\cdot"} which is often used when the decimal
mark is a comma.}
}
\value{
A character vector in which numbers are formatted in power-of-ten
notation in engineering form and delimited for rendering as inline equations
in an R markdown document.
}
\description{
Convert a numeric vector to a character vector in which the numbers are
formatted in power-of-ten notation in engineering form and delimited for
rendering as inline equations in an R markdown document.
}
\details{
In engineering notation, all exponents are multiples of three.
\code{format_engr()} is a wrapper for the more general function \code{format_numbers()}.
Where defaults are defined by \code{formatdown_options()}, users may reassign
the arguments locally in the function call or globally using
\code{formatdown_options()}.

Arguments after the dots (\code{...}) must be referred to by name.
}
\examples{
# input: single number
x <- 6.0221E+23
format_engr(x)

# input: units class
x <- 103400
units(x) <- "N m2 C-2"
format_engr(x)

# input: vector
data("metals", package = "formatdown")
x <- metals$dens
format_engr(x)

# significant digits
x <- 9.75358e+5
format_engr(x, 2)
format_engr(x, 3)
format_engr(x, 4)

# input: data frame
x <- metals[, c("thrm_exp", "thrm_cond")]
as.data.frame(apply(x, 2, format_engr, digits = 3))

# format_engr() same as format_numbers(..., format = "engr")
x <- 6.0221E+23
format_engr(x)
format_numbers(x, format = "engr")

# omit_power
x <- 103400
format_engr(x, omit_power = c(-1, 2)) # default
format_engr(x, omit_power = c(-1, 5))
format_engr(x, omit_power = 5) # equivalent to omit_power = c(5, 5)

# omit_power = NULL, power-of-ten notation for all elements
x <- c(1.2, 103400)
format_engr(x)
format_engr(x, omit_power = NULL)

# omit_power applies to native exponent (before engr formatting)
x <- 103400
format_sci(x) # native exponent is 5
format_engr(x, omit_power = 5)

# omit_power applies to exponent after engr formatting
x <- 103400
format_engr(x) # engr exponent is 3
format_engr(x, omit_power = 3)

# set_power overrides default engineering exponent
x <- 103400
format_engr(x)
format_engr(x, set_power = 4)

# set_power overrides omit_power
x <- 103400
format_engr(x)
format_engr(x, omit_power = 3)
format_engr(x, omit_power = 3, set_power = 3)
}
\seealso{
Other format_*: 
\code{\link{format_dcml}()},
\code{\link{format_nucl}()},
\code{\link{format_numbers}()},
\code{\link{format_sci}()},
\code{\link{format_text}()}
}
\concept{format_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{water}
\alias{water}
\title{Properties of water}
\format{
Classes data.table and data.frame with 11 observations
of 5 variables:

\describe{
\item{temp}{Numeric, temperature (K)}
\item{dens}{Numeric, density (kg m\eqn{^{-3}}{^{-3}})}
\item{sp_wt}{Numeric, specific weight (N m\eqn{^{-3}}{^{-3}})}
\item{visc}{Numeric, dynamic viscosity (Pa s)}
\item{bulk_mod}{Numeric, bulk modulus (Pa)}
}
}
\source{
E. Maurer E and I. Embry (2022) \emph{hydraulics: Basic Pipe and
Open Channel Hydraulics}, R package ver. 0.6.0,
\url{https://edm44.github.io/hydraulics/}.
}
\usage{
data(water, package = "formatdown")
}
\description{
Table of water properties at atmospheric pressure as a function of
temperature.
}
\keyword{datasets}
